@@ -573,7 +573,7 @@ free_and_return:
 	if (targetEntry) slapi_entry_free(targetEntry);
 	if (ber) ber_free(ber, 1);
 
-	LOG(errMesg ? errMesg : "success");
+	LOG("%s", errMesg ? errMesg : "success");
 	slapi_send_ldap_result(pb, rc, NULL, errMesg, 0, NULL);
 
 	return SLAPI_PLUGIN_EXTENDED_SENT_RESULT;
@@ -719,7 +719,7 @@ static int ipapwd_setkeytab(Slapi_PBlock *pb, struct ipapwd_krbcfg *krbcfg)
 	bsdn = slapi_be_getsuffix(be, 0);
 	if (bsdn == NULL) {
 		LOG_TRACE("Search for Base DN failed\n");
-		errMesg = "PrincipalName not found.\n";
+		errMesg = "PrincipalName not found (search for Base DN failed).\n";
 		rc = LDAP_NO_SUCH_OBJECT;
 		goto free_and_return;
 	}
@@ -743,7 +743,7 @@ static int ipapwd_setkeytab(Slapi_PBlock *pb, struct ipapwd_krbcfg *krbcfg)
 	if (ret == -1 || res != LDAP_SUCCESS) {
 		LOG_TRACE("Search for Principal failed, err (%d)\n",
 			  res ? res : ret);
-		errMesg = "PrincipalName not found.\n";
+		errMesg = "PrincipalName not found (search failed).\n";
 		rc = LDAP_NO_SUCH_OBJECT;
 		goto free_and_return;
 	}
@@ -752,7 +752,7 @@ static int ipapwd_setkeytab(Slapi_PBlock *pb, struct ipapwd_krbcfg *krbcfg)
 	slapi_pblock_get(pbte, SLAPI_PLUGIN_INTOP_SEARCH_ENTRIES, &es);
 	if (!es) {
 		LOG_TRACE("No entries ?!");
-		errMesg = "PrincipalName not found.\n";
+		errMesg = "PrincipalName not found (no result returned).\n";
 		rc = LDAP_NO_SUCH_OBJECT;
 		goto free_and_return;
 	}
@@ -763,7 +763,10 @@ static int ipapwd_setkeytab(Slapi_PBlock *pb, struct ipapwd_krbcfg *krbcfg)
 	/* if there is none or more than one, freak out */
 	if (i != 1) {
 		LOG_TRACE("Too many entries, or entry no found (%d)", i);
-		errMesg = "PrincipalName not found.\n";
+		if (i == 0)
+			errMesg = "PrincipalName not found.\n";
+		else
+			errMesg = "PrincipalName not found (too many entries).\n";
 		rc = LDAP_NO_SUCH_OBJECT;
 		goto free_and_return;
 	}
@@ -857,7 +860,7 @@ static int ipapwd_setkeytab(Slapi_PBlock *pb, struct ipapwd_krbcfg *krbcfg)
 		rtag = ber_scanf(ber, "{t[{t[i]t[o]}]", &ttmp, &ttmp, &tint, &ttmp, &tval);
 		if (rtag == LBER_ERROR) {
 			LOG_FATAL("ber_scanf failed\n");
-			errMesg = "Invalid payload, failed to decode.\n";
+			errMesg = "Invalid payload, failed to decode EncryptionKey.\n";
 			rc = LDAP_PROTOCOL_ERROR;
 			goto free_and_return;
 		}
@@ -906,7 +909,7 @@ static int ipapwd_setkeytab(Slapi_PBlock *pb, struct ipapwd_krbcfg *krbcfg)
 			rtag = ber_scanf(ber, "t[{t[i]", &ttmp, &ttmp, &tint);
 			if (rtag == LBER_ERROR) {
 				LOG_FATAL("ber_scanf failed\n");
-				errMesg = "Invalid payload, failed to decode.\n";
+				errMesg = "Invalid payload, failed to decode KrbSalt type.\n";
 				rc = LDAP_PROTOCOL_ERROR;
 				goto free_and_return;
 			}
@@ -920,7 +923,7 @@ static int ipapwd_setkeytab(Slapi_PBlock *pb, struct ipapwd_krbcfg *krbcfg)
 				rtag = ber_scanf(ber, "t[o]}]", &ttmp, &tval);
 				if (rtag == LBER_ERROR) {
 					LOG_FATAL("ber_scanf failed\n");
-					errMesg = "Invalid payload, failed to decode.\n";
+					errMesg = "Invalid payload, failed to decode KrbSalt contents.\n";
 					rc = LDAP_PROTOCOL_ERROR;
 					goto free_and_return;
 				}
@@ -948,7 +951,7 @@ static int ipapwd_setkeytab(Slapi_PBlock *pb, struct ipapwd_krbcfg *krbcfg)
 		}
 		if (rtag == LBER_ERROR) {
 			LOG_FATAL("ber_scanf failed\n");
-			errMesg = "Invalid payload, failed to decode.\n";
+			errMesg = "Invalid payload, failed to decode s2kparams.\n";
 			rc = LDAP_PROTOCOL_ERROR;
 			goto free_and_return;
 		}
@@ -1143,7 +1146,7 @@ free_and_return:
 
         if (rc == LDAP_SUCCESS)
             errMesg = NULL;
-	LOG(errMesg ? errMesg : "success");
+	LOG("%s", errMesg ? errMesg : "success");
 	slapi_send_ldap_result(pb, rc, NULL, errMesg, 0, NULL);
 
 	return SLAPI_PLUGIN_EXTENDED_SENT_RESULT;
@@ -1170,7 +1173,7 @@ static int ipapwd_extop(Slapi_PBlock *pb)
 	if (slapi_pblock_get(pb, SLAPI_EXT_OP_REQ_OID, &oid) != 0) {
 		errMesg = "Could not get OID value from request.\n";
 		rc = LDAP_OPERATIONS_ERROR;
-		LOG(errMesg);
+		LOG("%s", errMesg);
 		goto free_and_return;
 	} else {
 	        LOG("Received extended operation request with OID %s\n", oid);
@@ -1193,7 +1196,7 @@ static int ipapwd_extop(Slapi_PBlock *pb)
 free_and_return:
 	if (krbcfg) free_ipapwd_krbcfg(&krbcfg);
 
-	LOG(errMesg);
+	LOG("%s", errMesg);
 	slapi_send_ldap_result(pb, rc, NULL, errMesg, 0, NULL);
 
 	return SLAPI_PLUGIN_EXTENDED_SENT_RESULT;
@@ -1220,11 +1223,12 @@ Slapi_Filter *ipapwd_string2filter(char *strfilter)
 }
 
 /* Loads the OTP config entry, parses it, and applies it. */
-static inline bool ipapwd_load_otp_config(void)
+static bool ipapwd_load_otp_config(void)
 {
     char *config_attrs[] = { IPA_USER_AUTH_TYPE, NULL };
     Slapi_Entry *config_entry = NULL;
     Slapi_DN *config_sdn = NULL;
+    int ret;
 
     /* If we are using an alternate config area, check it for our
      * configuration, otherwise we just use our main plug-in config
@@ -1238,8 +1242,12 @@ static inline bool ipapwd_load_otp_config(void)
                     config_sdn ? slapi_sdn_get_ndn(config_sdn) : "null");
 
     /* Fetch the config entry. */
-    slapi_search_internal_get_entry(config_sdn, config_attrs, &config_entry,
-                                    ipapwd_plugin_id);
+    ret = slapi_search_internal_get_entry(config_sdn, config_attrs,
+                                          &config_entry, ipapwd_plugin_id);
+    if (ret != LDAP_SUCCESS) {
+        LOG_TRACE("Search for OTP config failed, err (%d)\n", ret);
+        /* fall through, defaults will be set */
+    }
 
     /* Parse and apply the config. */
     ipapwd_parse_otp_config_entry(config_entry, true);
