@@ -23,27 +23,14 @@ Password migration script
 import cgi
 import errno
 import glob
-import ldap
 import wsgiref
-from ipapython.ipa_log_manager import *
+
+from ipapython.ipa_log_manager import root_logger
 from ipapython.ipautil import get_ipa_basedn
 from ipapython.dn import DN
+from ipapython.ipaldap import IPAdmin
+from ipalib import errors
 
-BASE_DN = ''
-LDAP_URI = 'ldaps://localhost:636'
-
-def convert_exception(error):
-    """
-    Convert an LDAP exception into something more readable.
-    """
-    if not isinstance(error, ldap.TIMEOUT):
-        desc = error.args[0]['desc'].strip()
-        info = error.args[0].get('info', '').strip()
-    else:
-        desc = ''
-        info = ''
-
-    return '%s (%s)' % (desc, info)
 
 def wsgi_redirect(start_response, loc):
     start_response('302 Found', [('Location', loc)])
@@ -56,48 +43,45 @@ def get_ui_url(environ):
         raise ValueError('Cannot strip the script URL from full URL "%s"' % full_url)
     return full_url[:index] + "/ipa/ui"
 
-def get_base_dn():
+
+def get_base_dn(ldap_uri):
     """
     Retrieve LDAP server base DN.
     """
-    global BASE_DN
-
-    if BASE_DN:
-        return BASE_DN
     try:
-        conn = ldap.initialize(LDAP_URI)
-        conn.simple_bind_s('', '')
-        BASE_DN = get_ipa_basedn(conn)
-    except ldap.LDAPError, e:
+        conn = IPAdmin(ldap_uri=ldap_uri)
+        conn.do_simple_bind(DN(), '')
+        base_dn = get_ipa_basedn(conn)
+    except Exception, e:
         root_logger.error('migration context search failed: %s' % e)
         return ''
     finally:
-        conn.unbind_s()
+        conn.unbind()
 
-    return BASE_DN
+    return base_dn
 
-def bind(username, password):
-    base_dn = get_base_dn()
+
+def bind(ldap_uri, base_dn, username, password):
     if not base_dn:
         root_logger.error('migration unable to get base dn')
         raise IOError(errno.EIO, 'Cannot get Base DN')
     bind_dn = DN(('uid', username), ('cn', 'users'), ('cn', 'accounts'), base_dn)
     try:
-        conn = ldap.initialize(LDAP_URI)
-        conn.simple_bind_s(str(bind_dn), password)
-    except (ldap.INVALID_CREDENTIALS, ldap.UNWILLING_TO_PERFORM,
-            ldap.NO_SUCH_OBJECT), e:
-        root_logger.error('migration invalid credentials for %s: %s' % (bind_dn, convert_exception(e)))
-        raise IOError(errno.EPERM, 'Invalid LDAP credentials for user %s' % username)
-    except ldap.LDAPError, e:
-        root_logger.error('migration bind failed: %s' % convert_exception(e))
+        conn = IPAdmin(ldap_uri=ldap_uri)
+        conn.do_simple_bind(bind_dn, password)
+    except (errors.ACIError, errors.DatabaseError, errors.NotFound), e:
+        root_logger.error(
+            'migration invalid credentials for %s: %s' % (bind_dn, e))
+        raise IOError(
+            errno.EPERM, 'Invalid LDAP credentials for user %s' % username)
+    except Exception, e:
+        root_logger.error('migration bind failed: %s' % e)
         raise IOError(errno.EIO, 'Bind error')
     finally:
-        conn.unbind_s()
+        conn.unbind()
+
 
 def application(environ, start_response):
-    global LDAP_URI
-
     if environ.get('REQUEST_METHOD', None) != 'POST':
         return wsgi_redirect(start_response, 'index.html')
 
@@ -107,10 +91,15 @@ def application(environ, start_response):
 
     slapd_sockets = glob.glob('/var/run/slapd-*.socket')
     if slapd_sockets:
-        LDAP_URI = 'ldapi://%s' % slapd_sockets[0].replace('/', '%2f')
+        ldap_uri = 'ldapi://%s' % slapd_sockets[0].replace('/', '%2f')
+    else:
+        ldap_uri = 'ldaps://localhost:636'
+
+    base_dn = get_base_dn(ldap_uri)
 
     try:
-        bind(form_data['username'].value, form_data['password'].value)
+        bind(ldap_uri, base_dn,
+             form_data['username'].value, form_data['password'].value)
     except IOError as err:
         if err.errno == errno.EPERM:
             return wsgi_redirect(start_response, 'invalid.html')
