@@ -5,9 +5,10 @@
  * Copyright (C) 2010 Red Hat
  * see file 'COPYING' for use and warranty information
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; version 2 only
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,8 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 define([
@@ -483,8 +483,9 @@ IPA.attributes_widget = function(spec) {
         }).appendTo(container);
 
         that.$node = that.table = $('<table/>', {
-            id:id,
-            'class':'search-table aci-attribute-table scrollable'
+            id: id,
+            name: that.name,
+            'class': 'search-table aci-attribute-table scrollable'
         }).
             append('<thead/>').
             append('<tbody/>').
@@ -526,7 +527,7 @@ IPA.attributes_widget = function(spec) {
 
             var td =  $('<td/>').appendTo(tr);
             var name = that.get_input_name();
-            var id = that.option_next_id + name;
+            var id = that._option_next_id + name;
             td.append($('<input/>',{
                 id: id,
                 type: 'checkbox',
