@@ -530,7 +530,6 @@ IPA.association_table_widget = function (spec) {
             $('.action-button', that.table).addClass('action-button-disabled');
             that.unselect_all();
         }
-        that.enabled = enabled;
     };
 
     that.select_changed = function() {
@@ -609,10 +608,13 @@ IPA.association_table_widget = function (spec) {
         dialog.execute = function() {
             that.add(
                 dialog.get_selected_values(),
-                function() {
+                function(data) {
                     that.refresh();
                     dialog.close();
-                    IPA.notify_success('@i18n:association.added');
+
+                    var succeeded = IPA.get_succeeded(data);
+                    var msg = text.get('@i18n:association.added').replace('${count}', succeeded);
+                    IPA.notify_success(msg);
                 },
                 function() {
                     that.refresh();
@@ -671,9 +673,12 @@ IPA.association_table_widget = function (spec) {
         dialog.execute = function() {
             that.remove(
                 selected_values,
-                function() {
+                function(data) {
                     that.refresh();
-                    IPA.notify_success('@i18n:association.removed');
+
+                    var succeeded = IPA.get_succeeded(data);
+                    var msg = text.get('@i18n:association.removed').replace('${count}', succeeded);
+                    IPA.notify_success(msg);
                 },
                 function() {
                     that.refresh();
@@ -1084,17 +1089,8 @@ exp.association_facet = IPA.association_facet = function (spec, no_init) {
                 on_success: function(data) {
                     that.refresh();
                     dialog.close();
-                    var succeeded = data.result.completed;
-
-                    if (!succeeded) {
-                        succeeded = 0;
-                        for (var i = 0; i< data.result.results.length; i++) {
-                            if (data.result.results[i].completed === 1) {
-                                succeeded++;
-                            }
-                        }
-                    }
 
+                    var succeeded = IPA.get_succeeded(data);
                     var msg = text.get('@i18n:association.added').replace('${count}', succeeded);
                     IPA.notify_success(msg);
                 },
@@ -1148,17 +1144,7 @@ exp.association_facet = IPA.association_facet = function (spec, no_init) {
                 on_success: function(data) {
                     that.refresh();
 
-                    var succeeded = data.result.completed;
-
-                    if (!succeeded) {
-                        succeeded = 0;
-                        for (var i = 0; i< data.result.results.length; i++) {
-                            if (data.result.results[i].completed === 1) {
-                                succeeded++;
-                            }
-                        }
-                    }
-
+                    var succeeded = IPA.get_succeeded(data);
                     var msg = text.get('@i18n:association.removed').replace('${count}', succeeded);
                     IPA.notify_success(msg);
                 },
@@ -1417,7 +1403,10 @@ exp.attribute_facet = IPA.attribute_facet = function(spec, no_init) {
                 function(data) {
                     that.load(data);
                     that.show_content();
-                    IPA.notify_success('@i18n:association.removed');
+
+                    var succeeded = IPA.get_succeeded(data);
+                    var msg = text.get('@i18n:association.removed').replace('${count}', succeeded);
+                    IPA.notify_success(msg);
                 },
                 function() {
                     that.refresh();
@@ -1466,53 +1455,6 @@ exp.attribute_facet = IPA.attribute_facet = function(spec, no_init) {
     return that;
 };
 
-IPA.sid_facet = function(spec, no_init) {
-
-    spec.name = spec.name || 'sid_facet';
-
-    var that = IPA.attribute_facet(spec, no_init);
-
-    that.load_records = function(value) {
-        var xlate = {};
-        var sidxlate_command = IPA.command({
-            entity: 'trust',
-            method: 'resolve',
-            options: {
-                sids: ''
-            }
-        });
-        sidxlate_command.on_success = function(data, text_status, xhr) {
-            for (var i=0; i< data.result.result.length; i++) {
-                var entry = data.result.result[i];
-                if (entry.sid[0] in xlate) {
-                    xlate[entry.sid[0]].resolve(entry.name[0]);
-                }
-            }
-        };
-        that.table.empty();
-
-        if (value.length === 0) return;
-
-        var sids = [];
-        for (var i=0; i< value.length; i++) {
-            var sid = value[i][that.attribute];
-            var deferred = new Deferred();
-            value[i][that.attribute] = {
-                promise: deferred.promise,
-                temp: sid
-            };
-            xlate[sid] = deferred;
-            sids.push(sid);
-            that.add_record(value[i]);
-        }
-        sidxlate_command.options.sids = sids;
-        sidxlate_command.execute();
-    };
-
-    return that;
-};
-
-
 IPA.attr_read_only_evaluator = function(spec) {
 
     spec.name = spec.name || 'attr_read_only_evaluator';
