@@ -82,8 +82,7 @@ return {
                                     label: '@i18n:objects.dnsconfig.forward_none'
                                 }
                             ]
-                        },
-                        'idnszonerefresh'
+                        }
                     ]
                 }
             ],
@@ -446,14 +445,14 @@ IPA.dnszone_name_section = function(spec) {
             title: idnsname.label
         }).appendTo(tr);
 
-        var span = $('<span/>', {
+        var widget_cont = $('<div/>', {
             name: 'idnsname',
             'class': 'field'
         }).appendTo(td);
 
-        idnsname.create(span);
+        idnsname.create(widget_cont);
 
-        var idnsname_input = $('input', span);
+        var idnsname_input = $('input', widget_cont);
 
         var name_from_ip = that.widgets.get_widget('name_from_ip');
 
@@ -481,12 +480,12 @@ IPA.dnszone_name_section = function(spec) {
             title: name_from_ip.label
         }).appendTo(tr);
 
-        span = $('<span/>', {
+        widget_cont = $('<div/>', {
             name: 'name_from_ip',
             'class': 'field'
         }).appendTo(td);
 
-        name_from_ip.create(span);
+        name_from_ip.create(widget_cont);
 
         idnsname.radio.click();
     };
@@ -603,7 +602,7 @@ IPA.dnszone_adder_dialog = function(spec) {
         var zone = zone_w.save()[0] || '';
         var ns = ns_w.save()[0] || '';
 
-        var zone_is_reverse = !zone_w.is_enabled() ||
+        var zone_is_reverse = !zone_w.enabled ||
                               ends_with(zone, '.in-addr.arpa.') ||
                               ends_with(zone, '.ip6.arpa.');
         var relative_ns = true;
@@ -1314,7 +1313,7 @@ IPA.dnsrecord_redirection_dialog = function(spec) {
     };
 
     that.on_ok = function() {
-        navigation.show_entity_page('dnszone','default');
+        navigation.show_entity('dnszone','default');
     };
 
     return that;
@@ -1767,7 +1766,6 @@ IPA.dns.record_type_table_widget = function(spec) {
             $('.action-button', that.table).addClass('action-button-disabled');
             that.unselect_all();
         }
-        that.enabled = enabled;
     };
 
     that.select_changed = function() {
