@@ -352,7 +352,7 @@ exp.facet = IPA.facet = function(spec, no_init) {
             that.domNode.detach();
         } else {
             that.domNode = $('<div/>', {
-                'class': 'facet',
+                'class': 'facet active-facet',
                 name: that.name,
                 'data-name': that.name,
                 'data-entity': entity_name
@@ -386,8 +386,7 @@ exp.facet = IPA.facet = function(spec, no_init) {
         }).appendTo(domNode);
 
         that.create_content(that.content);
-
-
+        domNode.removeClass('active-facet');
     };
 
     that.create_header = function(container) {
@@ -429,7 +428,7 @@ exp.facet = IPA.facet = function(spec, no_init) {
                 that.clear();
             }
 
-            that.domNode.css('display', 'block');
+            that.domNode.addClass('active-facet');
             that.show_content();
             that.header.select_tab();
 
@@ -437,7 +436,7 @@ exp.facet = IPA.facet = function(spec, no_init) {
                 that.refresh();
             }
         } else {
-            that.domNode.css('display', 'block');
+            that.domNode.addClass('active-facet');
             that.show_content();
             that.header.select_tab();
         }
@@ -459,7 +458,7 @@ exp.facet = IPA.facet = function(spec, no_init) {
     };
 
     that.hide = function() {
-        that.domNode.css('display', 'none');
+        that.domNode.removeClass('active-facet');
     };
 
     that.load = function(data) {
@@ -1083,7 +1082,6 @@ exp.table_facet = IPA.table_facet = function(spec, no_init) {
 
     that.pagination = spec.pagination === undefined ? true : spec.pagination;
     that.search_all_entries = spec.search_all_entries;
-    that.search_all_attributes = spec.search_all_attributes;
     that.sort_enabled = spec.sort_enabled === undefined ? true : spec.sort_enabled;
     that.selectable = spec.selectable === undefined ? true : spec.selectable;
     that.select_changed = IPA.observer();
@@ -1313,7 +1311,7 @@ exp.table_facet = IPA.table_facet = function(spec, no_init) {
 
     that.create_get_records_command = function(pkeys, on_success, on_error) {
 
-         var batch = IPA.batch_command({
+        var batch = IPA.batch_command({
             name: that.get_records_command_name(),
             on_success: on_success,
             on_error: on_error
@@ -1325,10 +1323,13 @@ exp.table_facet = IPA.table_facet = function(spec, no_init) {
             var command = IPA.command({
                 entity: that.table.entity.name,
                 method: 'show',
-                args: [ pkey ],
-                options: { all: true }
+                args: [pkey]
             });
 
+            if (that.table.entity.has_members()) {
+                command.set_options({no_members: true});
+            }
+
             batch.add_command(command);
         }
 
@@ -1354,7 +1355,6 @@ exp.table_facet = IPA.table_facet = function(spec, no_init) {
             label: entity.metadata.label,
             entity: entity,
             pagination: true,
-            search_all_attributes: that.search_all_attributes,
             scrollable: true,
             selectable: that.selectable && !that.read_only
         });
@@ -1940,7 +1940,7 @@ exp.action_button_widget = IPA.action_button_widget = function(spec) {
     };
 
     that.set_enabled = function(enabled) {
-        that.enabled = enabled;
+        that.widget_set_enabled(enabled);
 
         if (that.button_element) {
             if (enabled) {
