@@ -478,7 +478,7 @@ IPA.update_password_expiration = function() {
 
 IPA.password_selfservice = function() {
     var reset_dialog = IPA.user_password_dialog({
-        self_service: true,
+        pkey: IPA.whoami.uid[0],
         on_success: function() {
             var command = IPA.get_whoami_command();
             var orig_on_success = command.on_success;
@@ -488,7 +488,7 @@ IPA.password_selfservice = function() {
             };
             command.execute();
 
-            alert(text.get('@i18n:password.password_change_complete'));
+            IPA.notify_success(text.get('@i18n:password.password_change_complete'));
             reset_dialog.close();
         }
     });
@@ -1232,6 +1232,7 @@ IPA.error_dialog = function(spec) {
     spec = spec || {};
 
     spec.id = spec.id || 'error_dialog';
+    spec.name = 'error_dialog';
     spec.title = spec.error_thrown.name;
 
     var that = IPA.dialog(spec);
@@ -1949,6 +1950,21 @@ IPA.notify_success = function(message, timeout) {
     }, timeout || IPA.config.message_timeout);
 };
 
+IPA.get_succeeded = function(data) {
+    var succeeded = data.result.completed;
+
+    if (typeof succeeded !== 'number') {
+        succeeded = 0;
+        for (var i = 0; i< data.result.results.length; i++) {
+            if (data.result.results[i].completed === 1) {
+                succeeded++;
+            }
+        }
+    }
+
+    return succeeded;
+};
+
 IPA.config = {
     default_priority: 500,
     message_timeout: 3000, // [ms]
