@@ -4,9 +4,10 @@
  * Copyright (C) 2010 Red Hat
  * see file 'COPYING' for use and warranty information
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; version 2 only
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,10 +15,22 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+define([
+        'freeipa/metadata',
+        'freeipa/aci',
+        'freeipa/details',
+        'freeipa/facet',
+        'freeipa/field',
+        'freeipa/ipa',
+        'freeipa/jquery',
+        'freeipa/reg',
+        'freeipa/widget'
+       ],
+        function(md, aci, mod_details, mod_facet, fields, IPA, $, reg, widgets) {
+    return function() {
 
 var target_container;
 var target_widget;
@@ -27,6 +40,13 @@ var group_entity = IPA.entity({ name: 'group' });
 
 module('aci', {
         setup: function() {
+
+            fields.register();
+            widgets.register();
+            aci.register();
+            mod_facet.register();
+            mod_details.register();
+
             IPA.ajax_options.async = false;
             IPA.init({
                 url: 'data',
@@ -39,7 +59,7 @@ module('aci', {
                 entity: entity,
                 fields: [
                     {
-                        type: 'select',
+                        $type: 'select',
                         name: 'target',
                         widget: 'target.target',
                         enabled: false
@@ -50,7 +70,7 @@ module('aci', {
                         enabled: false
                     },
                     {
-                        type: 'entity_select',
+                        $type: 'entity_select',
                         name: 'memberof',
                         widget: 'target.memberof',
                         enabled: false
@@ -61,13 +81,13 @@ module('aci', {
                         enabled: false
                     },
                     {
-                        type: 'entity_select',
+                        $type: 'entity_select',
                         name: 'targetgroup',
                         widget: 'target.targetgroup',
                         enabled: false
                     },
                     {
-                        type: 'select',
+                        $type: 'select',
                         name: 'type',
                         widget: 'target.type',
                         enabled: false
@@ -80,14 +100,14 @@ module('aci', {
                     {
                         name: 'attrs_multi',
                         param: 'attrs',
-                        type: 'multivalued',
+                        $type: 'multivalued',
                         widget: 'target.attrs_multi',
                         enabled: false
                     }
                 ],
                 widgets: [
                     {
-                        type: 'permission_target',
+                        $type: 'permission_target',
                         container_factory: IPA.details_table_section,
                         group_entity: group_entity,
                         name: 'target',
@@ -96,13 +116,16 @@ module('aci', {
                     }
                 ],
                 policies: [
-                    IPA.permission_target_policy('target')
+                    {
+                        $factory: IPA.permission_target_policy,
+                        widget_name: 'target'
+                    }
                 ]
             });
-           entity.add_facet('details', target_facet);
-
-            target_container = $('<div id="target"/>').appendTo(document.body);
-            target_facet.create(target_container);
+            entity.add_facet('details', target_facet);
+            target_container = $('<div id="content"/>').appendTo(document.body);
+            target_facet.container_node = target_container[0];
+            target_facet.create();
             target_widget = target_facet.widgets.get_widget('target');
         },
         teardown: function() {
@@ -113,7 +136,7 @@ module('aci', {
 
 test("IPA.attributes_widget.", function() {
 
-    var aciattrs = IPA.metadata.objects['user'].aciattrs;
+    var aciattrs = md.source.objects.user.aciattrs;
 
     var container = $('<span/>', {
         name: 'attrs'
@@ -286,5 +309,4 @@ test("Testing subtree target.", function() {
     same(get_visible_rows(target_widget), ['memberof', 'subtree', 'attrs_multi'], 'subtree row visible');
 });
 
-
-
+};});
\ No newline at end of file
