@@ -18,6 +18,19 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+define([
+    'freeipa/metadata',
+    'freeipa/ipa',
+    'freeipa/jquery',
+    'freeipa/details',
+    'freeipa/facet',
+    'freeipa/reg',
+    'freeipa/entity',
+    'freeipa/field',
+    'freeipa/widget'],
+        function(md, IPA, $, mod_details, mod_facet, reg) {
+    return function() {
+
 var details_container;
 
 
@@ -25,6 +38,9 @@ module('details', {
     setup: function() {
         IPA.ajax_options.async = false;
 
+        mod_facet.register();
+        mod_details.register();
+
         IPA.init({
             url: 'data',
             on_error: function(xhr, text_status, error_thrown) {
@@ -32,24 +48,19 @@ module('details', {
             }
         });
 
-        IPA.nav = {};
-
-        IPA.nav.get_state = function(key){
-            return $.bbq.getState(key);
-        };
-
         details_container = $('<div id="details"/>').appendTo(document.body);
 
         IPA.register('user', function(spec) {
 
             return IPA.entity({
                 name: 'user',
-                metadata: IPA.metadata.objects.user
+                metadata: md.source.objects.user
             });
         });
     },
     teardown: function() {
         details_container.remove();
+        reg.facet.remove('details');
     }
 });
 
@@ -136,16 +147,6 @@ test("Testing details lifecycle: create, load.", function(){
     var update_success_called = false;
     var update_failure_called = false;
 
-
-    function save_password(){
-        save_called = true;
-        return [];
-    }
-
-    function load_manager(){
-        load_called = true;
-    }
-
     function test_field(spec) {
         var that = IPA.field(spec);
 
@@ -170,8 +171,8 @@ test("Testing details lifecycle: create, load.", function(){
         return that;
     }
 
-    IPA.field_factories['test'] = test_field;
-    IPA.widget_factories['test'] = test_widget;
+    reg.field.register('test', test_field);
+    reg.widget.register('test', test_widget);
 
     IPA.register('user', function(spec) {
 
@@ -192,27 +193,27 @@ test("Testing details lifecycle: create, load.", function(){
                         label: 'contact',
                         fields: [
                             {
-                                type: 'test',
+                                $type: 'test',
                                 name:'test'
                             },
                             {
-                                type: 'multivalued',
+                                $type: 'multivalued',
                                 name:'mail'
                             },
                             {
-                                type: 'multivalued',
+                                $type: 'multivalued',
                                 name:'telephonenumber'
                             },
                             {
-                                type: 'multivalued',
+                                $type: 'multivalued',
                                 name:'pager'
                             },
                             {
-                                type: 'multivalued',
+                                $type: 'multivalued',
                                 name:'mobile'
                             },
                             {
-                                type: 'multivalued',
+                                $type: 'multivalued',
                                 name:'facsimiletelephonenumber'
                             }
                         ]
@@ -225,33 +226,20 @@ test("Testing details lifecycle: create, load.", function(){
     });
 
     var entity = IPA.get_entity('user');
-
-    var entity_container = $('<div/>', {
-        name: 'user',
-        title: 'User',
-        'class': 'entity'
-    }).appendTo(details_container);
-
-    entity.create(entity_container);
-
+    var container = $('<div/>', {}).appendTo(details_container);
     var facet = entity.get_facet('details');
-
-    var facet_container = $('<div/>', {
-        name: facet.name,
-        'class': 'facet'
-    });
-
-    facet.create(facet_container);
+    facet.container_node = container[0];
+    facet.create();
 
     facet.load(data);
 
-    var contact = $('.details-section[name=contact]', facet_container);
+    var contact = $('.details-section[name=contact]', facet.domNode);
 
     ok(
         contact.length,
         'Verifying section for contact is created');
 
-    var identity = $('.details-section[name=identity]', facet_container);
+    var identity = $('.details-section[name=identity]', facet.domNode);
 
     ok(
         identity.length,
@@ -263,8 +251,6 @@ test("Testing details lifecycle: create, load.", function(){
         rows.length, 6,
         'Verifying rows for identity');
 
-    facet_container.attr('id','user');
-
     ok (load_called, 'load manager called');
 
     var field = facet.fields.get_field('test');
@@ -352,3 +338,5 @@ test("Testing IPA.details_section_create again()",function() {
             'Verifying field '+field.name+' was created');
     }
 });
+
+};});
\ No newline at end of file
