@@ -50,7 +50,7 @@ from errors import (PublicError, CommandError, HelpError, InternalError,
         NoSuchNamespaceError, ValidationError, NotFound, NotConfiguredError,
         PromptFailed, ConversionError)
 from constants import CLI_TAB
-from parameters import Password, Bytes, File, Str, StrEnum
+from parameters import Password, Bytes, File, Str, StrEnum, Any
 from text import _
 from ipapython.version import API_VERSION
 
@@ -293,7 +293,7 @@ class textui(backend.Backend):
                     return
                 else:
                     if len(value) > 0:
-                        text = ', '.join(value)
+                        text = ', '.join(str(v) for v in value)
                     else:
                         return
                 line_len = self.get_tty_width()
@@ -664,7 +664,13 @@ class help(frontend.Local):
     Display help for a command or topic.
     """
 
-    takes_args = (Str('command?'),)
+    takes_args = (
+        Str('command?', cli_name='topic', label=_('Topic or Command'),
+            doc=_('The topic or command name.')),
+    )
+    takes_options = (
+        Any('outfile?', flags=['no_option']),
+    )
 
     has_output = tuple()
 
@@ -749,56 +755,56 @@ class help(frontend.Local):
 
         super(help, self)._on_finalize()
 
-    def run(self, key):
+    def run(self, key, outfile=None, **options):
+        if outfile is None:
+            outfile = sys.stdout
+        writer = self._writer(outfile)
         name = from_cli(key)
         mod_name = '%s.%s' % (self._PLUGIN_BASE_MODULE, name)
-        if key is None or name == "topics":
-            self.print_topics()
+        if key is None:
+            self.api.parser.print_help(outfile)
+            return
+        if name == "topics":
+            self.print_topics(outfile)
             return
         if name in self._topics:
-            self.print_commands(name)
+            self.print_commands(name, outfile)
         elif name in self.Command:
             cmd = self.Command[name]
             if cmd.NO_CLI:
                 raise HelpError(topic=name)
-            print unicode(_('Purpose: %s')) % unicode(_(cmd.doc)).strip()
-            self.Backend.cli.build_parser(cmd).print_help()
+            self.Backend.cli.build_parser(cmd).print_help(outfile)
         elif mod_name in sys.modules:
-            self.print_commands(name)
+            self.print_commands(name, outfile)
         elif name == "commands":
             mcl = max(len(s) for s in (self.Command))
             for cname in self.Command:
                 cmd = self.Command[cname]
                 if cmd.NO_CLI:
                     continue
-                print '%s  %s' % (to_cli(cmd.name).ljust(mcl), cmd.summary)
+                writer('%s  %s' % (to_cli(cmd.name).ljust(mcl), cmd.summary))
         else:
             raise HelpError(topic=name)
 
-    def print_topics(self):
-        topics = sorted(self._topics.keys())
+    def _writer(self, outfile):
+        def writer(string=''):
+            print >> outfile, unicode(string)
+        return writer
 
-        print unicode(_('Usage: ipa [global-options] COMMAND ...'))
-        print ''
-        print unicode(_('Built-in commands:'))
-        for c in self._builtins:
-            print unicode(_('Help subtopics:'))
-            print '  %s  %s' % (to_cli(c.name).ljust(self._mtl), c.summary)
-        print ''
-        print unicode(_('Help topics:'))
-        for t in topics:
-            topic = self._topics[t]
-            print '  %s  %s' % (to_cli(t).ljust(self._mtl), topic[0])
-        print ''
-        print unicode(_('Try `ipa --help` for a list of global options.'))
+    def print_topics(self, outfile):
+        writer = self._writer(outfile)
 
-    def print_commands(self, topic):
+        for t, topic in sorted(self._topics.items()):
+            writer('%s  %s' % (to_cli(t).ljust(self._mtl), topic[0]))
+
+    def print_commands(self, topic, outfile):
+        writer = self._writer(outfile)
         if topic in self._topics and type(self._topics[topic][2]) is dict:
             # we want to display topic which has subtopics
             for subtopic in self._topics[topic][2]:
                 doc = self._topics[topic][2][subtopic][0]
                 mcl = self._topics[topic][1]
-                print '  %s  %s' % (to_cli(subtopic).ljust(mcl), doc)
+                writer('  %s  %s' % (to_cli(subtopic).ljust(mcl), doc))
         else:
             # we want to display subtopic or a topic which has no subtopics
             if topic in self._topics:
@@ -821,13 +827,17 @@ class help(frontend.Local):
             if topic not in self.Command and len(commands) == 0:
                 raise HelpError(topic=topic)
 
-            print doc
+            writer(doc)
             if commands:
-                print ''
-                print unicode(_('Topic commands:'))
+                writer()
+                writer(_('Topic commands:'))
                 for c in commands:
-                    print '  %s  %s' % (to_cli(c.name).ljust(mcl), c.summary)
-            print "\n"
+                    writer(
+                        '  %s  %s' % (to_cli(c.name).ljust(mcl), c.summary))
+                writer()
+                writer(_('To get command help, use:'))
+                writer(_('  ipa <command> --help'))
+            writer()
 
 class show_mappings(frontend.Command):
     """
@@ -840,7 +850,7 @@ class show_mappings(frontend.Command):
     )
     has_output = tuple()
 
-    def run(self, command_name):
+    def run(self, command_name, **options):
         command_name = from_cli(command_name)
         if command_name not in self.Command:
             raise CommandError(name=command_name)
@@ -862,7 +872,7 @@ class console(frontend.Command):
 
     has_output = tuple()
 
-    def run(self):
+    def run(self, **options):
         code.interact(
             '(Custom IPA interactive Python console)',
             local=dict(api=self.api)
@@ -1013,14 +1023,15 @@ class cli(backend.Executioner):
         On incorrect invocation, prints out a help message and returns None
         """
         if len(argv) == 0:
-            self.Command.help()
-            return
+            self.Command.help(outfile=sys.stderr)
+            print >>sys.stderr
+            print >>sys.stderr, 'Error: Command not specified'
+            exit(2)
         (key, argv) = (argv[0], argv[1:])
         name = from_cli(key)
         if name not in self.Command and len(argv) == 0:
             try:
-                self.Command.help(unicode(key))
-                return
+                self.Command.help(unicode(key), outfile=sys.stderr)
             except HelpError:
                 pass
         if name not in self.Command or self.Command[name].NO_CLI:
@@ -1028,12 +1039,17 @@ class cli(backend.Executioner):
         cmd = self.Command[name]
         return cmd
 
-    def argv_to_keyword_arguments(self, cmd, argv):
+    def process_keyword_arguments(self, cmd, kw):
         """Get the keyword arguments for a Command"""
-        kw = self.parse(cmd, argv)
         if self.env.interactive:
             self.prompt_interactively(cmd, kw)
-        kw = cmd.split_csv(**kw)
+            try:
+                callbacks = cmd.get_callbacks('interactive_prompt')
+            except AttributeError:
+                pass
+            else:
+                for callback in callbacks:
+                    callback(cmd, kw)
         kw['version'] = API_VERSION
         self.load_files(cmd, kw)
         return kw
@@ -1043,10 +1059,11 @@ class cli(backend.Executioner):
         if cmd is None:
             return
         name = cmd.name
+        kw = self.parse(cmd, argv[1:])
         if not isinstance(cmd, frontend.Local):
             self.create_context()
         try:
-            kw = self.argv_to_keyword_arguments(cmd, argv[1:])
+            kw = self.process_keyword_arguments(cmd, kw)
             result = self.execute(name, **kw)
             if callable(cmd.output_for_cli):
                 for param in cmd.params():
@@ -1079,7 +1096,9 @@ class cli(backend.Executioner):
 
     def build_parser(self, cmd):
         parser = CLIOptionParser(
-            usage=' '.join(self.usage_iter(cmd))
+            usage=' '.join(self.usage_iter(cmd)),
+            description=unicode(cmd.doc),
+            formatter=IPAHelpFormatter(),
         )
         option_groups = {}
         for option in cmd.options():
@@ -1120,7 +1139,7 @@ class cli(backend.Executioner):
 
         for arg in cmd.args():
             name = self.__get_arg_name(arg, format_name=False)
-            if name is None:
+            if 'no_option' in arg.flags or name is None:
                 continue
             doc = unicode(arg.doc)
             parser.add_argument(name, doc)
@@ -1159,11 +1178,13 @@ class cli(backend.Executioner):
         ``self.env.prompt_all`` is ``True``, this method will prompt for any
         params that have a missing values, even if the param is optional.
         """
+
         honor_alwaysask = True
         for param in cmd.params():
             if param.alwaysask and param.name in kw:
                 honor_alwaysask = False
                 break
+
         for param in cmd.params():
             if (param.required and param.name not in kw) or \
                 (param.alwaysask and honor_alwaysask) or self.env.prompt_all:
@@ -1177,32 +1198,21 @@ class cli(backend.Executioner):
                     )
                 else:
                     default = cmd.get_default_of(param.name, **kw)
-                    error = None
-                    while True:
-                        if error is not None:
-                            self.Backend.textui.print_prompt_attribute_error(unicode(param.label),
-                                                                             unicode(error))
-                        raw = self.Backend.textui.prompt(param.label, default, optional=param.alwaysask or not param.required)
-                        try:
-                            value = param(raw, **kw)
-                            if value is not None:
-                                kw[param.name] = value
-                            break
-                        except (ValidationError, ConversionError), e:
-                            error = e.error
+                    optional = param.alwaysask or not param.required
+
+                    value = cmd.prompt_param(param,
+                                             default=default,
+                                             optional=optional,
+                                             kw=kw)
+
+                    if value is not None:
+                        kw[param.name] = value
+
             elif param.password and kw.get(param.name, False) is True:
                 kw[param.name] = self.Backend.textui.prompt_password(
                     param.label, param.confirm
                 )
 
-        try:
-            callbacks = cmd.get_callbacks('interactive_prompt')
-        except AttributeError:
-            pass
-        else:
-            for callback in callbacks:
-                callback(cmd, kw)
-
     def load_files(self, cmd, kw):
         """
         Load files from File parameters.
@@ -1246,6 +1256,21 @@ class cli(backend.Executioner):
                 kw[p.name] = self.Backend.textui.decode(raw)
 
 
+class IPAHelpFormatter(optparse.IndentedHelpFormatter):
+    """Formatter suitable for printing IPA command help
+
+    The default help formatter reflows text to fit the terminal, but it
+    ignores line/paragraph breaks.
+    IPA's descriptions already have correct line breaks. This formatter
+    doesn't touch them (save for removing initial/trailing whitespace).
+    """
+    def format_description(self, description):
+        if description:
+            return description.strip()
+        else:
+            return ""
+
+
 cli_plugins = (
     cli,
     textui,
