@@ -102,10 +102,9 @@ current block assignments:
             - **5100 - 5999**  *Reserved for future use*
 """
 
-from inspect import isclass
-from text import _ as ugettext, ngettext as ungettext
-from text import Gettext, NGettext
-from constants import TYPE_ERROR
+from ipalib.text import ngettext as ungettext
+
+import messages
 
 
 class PrivateError(StandardError):
@@ -166,7 +165,7 @@ class PluginSubclassError(PrivateError):
 
     """
 
-    format  = '%(plugin)r not subclass of any base in %(bases)r'
+    format = '%(plugin)r not subclass of any base in %(bases)r'
 
 
 class PluginDuplicateError(PrivateError):
@@ -233,10 +232,10 @@ class PluginsPackageError(PrivateError):
 ##############################################################################
 # Public errors:
 
-__messages = []
+_texts = []
 
 def _(message):
-    __messages.append(message)
+    _texts.append(message)
     return message
 
 
@@ -244,58 +243,14 @@ class PublicError(StandardError):
     """
     **900** Base class for exceptions that can be forwarded in an RPC response.
     """
+    def __init__(self, format=None, message=None, **kw):
+        messages.process_message_arguments(self, format, message, **kw)
+        super(PublicError, self).__init__(self.msg)
 
     errno = 900
     rval = 1
     format = None
 
-    def __init__(self, format=None, message=None, **kw):
-        self.kw = kw
-        name = self.__class__.__name__
-        if self.format is not None and format is not None:
-            raise ValueError(
-                'non-generic %r needs format=None; got format=%r' % (
-                    name, format)
-            )
-        if message is None:
-            if self.format is None:
-                if format is None:
-                    raise ValueError(
-                        '%s.format is None yet format=None, message=None' % name
-                    )
-                self.format = format
-            self.forwarded = False
-            self.msg = self.format % kw
-            if isinstance(self.format, basestring):
-                self.strerror = ugettext(self.format) % kw
-            else:
-                self.strerror = self.format % kw
-            if 'instructions' in kw:
-                def convert_instructions(value):
-                    if isinstance(value, list):
-                        result=u'\n'.join(map(lambda line: unicode(line), value))
-                        return result
-                    return value
-                instructions = u'\n'.join((unicode(_('Additional instructions:')),
-                                          convert_instructions(kw['instructions'])))
-                self.strerror = u'\n'.join((self.strerror, instructions))
-        else:
-            if isinstance(message, (Gettext, NGettext)):
-                message = unicode(message)
-            elif type(message) is not unicode:
-                raise TypeError(
-                    TYPE_ERROR % ('message', unicode, message, type(message))
-                )
-            self.forwarded = True
-            self.msg = message
-            self.strerror = message
-        for (key, value) in kw.iteritems():
-            assert not hasattr(self, key), 'conflicting kwarg %s.%s = %r' % (
-                name, key, value,
-            )
-            setattr(self, key, value)
-        StandardError.__init__(self, self.msg)
-
 
 class VersionError(PublicError):
     """
@@ -311,7 +266,7 @@ class VersionError(PublicError):
     """
 
     errno = 901
-    format = _('%(cver)s client incompatible with %(sver)s server at %(server)r')
+    format = _("%(cver)s client incompatible with %(sver)s server at '%(server)s'")
 
 
 class UnknownError(PublicError):
@@ -367,7 +322,7 @@ class ServerInternalError(PublicError):
     """
 
     errno = 904
-    format = _('an internal error has occurred on server at %(server)r')
+    format = _("an internal error has occurred on server at '%(server)s'")
 
 
 class CommandError(PublicError):
@@ -383,7 +338,7 @@ class CommandError(PublicError):
     """
 
     errno = 905
-    format = _('unknown command %(name)r')
+    format = _("unknown command '%(name)s'")
 
 
 class ServerCommandError(PublicError):
@@ -400,7 +355,7 @@ class ServerCommandError(PublicError):
     """
 
     errno = 906
-    format = _('error on server %(server)r: %(error)s')
+    format = _("error on server '%(server)s': %(error)s")
 
 
 class NetworkError(PublicError):
@@ -416,7 +371,7 @@ class NetworkError(PublicError):
     """
 
     errno = 907
-    format = _('cannot connect to %(uri)r: %(error)s')
+    format = _("cannot connect to '%(uri)s': %(error)s")
 
 
 class ServerNetworkError(PublicError):
@@ -425,7 +380,7 @@ class ServerNetworkError(PublicError):
     """
 
     errno = 908
-    format = _('error on server %(server)r: %(error)s')
+    format = _("error on server '%(server)s': %(error)s")
 
 
 class JSONError(PublicError):
@@ -526,7 +481,7 @@ class ServiceError(KerberosError):
     """
 
     errno = 1102
-    format = _('Service %(service)r not found in Kerberos database')
+    format = _("Service '%(service)s' not found in Kerberos database")
 
 
 class NoCCacheError(KerberosError):
@@ -688,7 +643,7 @@ class ZeroArgumentError(InvocationError):
     """
 
     errno = 3003
-    format = _('command %(name)r takes no arguments')
+    format = _("command '%(name)s' takes no arguments")
 
 
 class MaxArgumentError(InvocationError):
@@ -708,8 +663,8 @@ class MaxArgumentError(InvocationError):
     def __init__(self, message=None, **kw):
         if message is None:
             format = ungettext(
-                'command %(name)r takes at most %(count)d argument',
-                'command %(name)r takes at most %(count)d arguments',
+                "command '%(name)s' takes at most %(count)d argument",
+                "command '%(name)s' takes at most %(count)d arguments",
                 kw['count']
             )
         else:
@@ -738,7 +693,7 @@ class OverlapError(InvocationError):
     """
 
     errno = 3006
-    format = _('overlapping arguments and options: %(names)r')
+    format = _("overlapping arguments and options: %(names)s")
 
 
 class RequirementError(InvocationError):
@@ -754,7 +709,7 @@ class RequirementError(InvocationError):
     """
 
     errno = 3007
-    format = _('%(name)r is required')
+    format = _("'%(name)s' is required")
 
 
 class ConversionError(InvocationError):
@@ -770,7 +725,7 @@ class ConversionError(InvocationError):
     """
 
     errno = 3008
-    format = _('invalid %(name)r: %(error)s')
+    format = _("invalid '%(name)s': %(error)s")
 
 
 class ValidationError(InvocationError):
@@ -786,7 +741,7 @@ class ValidationError(InvocationError):
     """
 
     errno = 3009
-    format = _('invalid %(name)r: %(error)s')
+    format = _("invalid '%(name)s': %(error)s")
 
 
 class NoSuchNamespaceError(InvocationError):
@@ -802,7 +757,7 @@ class NoSuchNamespaceError(InvocationError):
     """
 
     errno = 3010
-    format = _('api has no such namespace: %(name)r')
+    format = _("api has no such namespace: '%(name)s'")
 
 
 class PasswordMismatch(InvocationError):
@@ -841,6 +796,21 @@ class PromptFailed(InvocationError):
     format = _('Could not get %(name)s interactively')
 
 
+class DeprecationError(InvocationError):
+    """
+    **3015** Raise when a command has been deprecated
+
+    For example:
+
+    >>> raise DeprecationError(name='hbacrule_add_sourcehost')
+    Traceback (most recent call last):
+      ...
+    DeprecationError: Command 'hbacrule_add_sourcehost' has been deprecated
+    """
+    errno = 3015
+    format = _("Command '%(name)s' has been deprecated")
+
+
 ##############################################################################
 # 4000 - 4999: Execution errors
 
@@ -978,7 +948,7 @@ class MalformedUserPrincipal(ExecutionError):
     """
 
     errno = 4008
-    format = _('Principal is not of the form user@REALM: %(principal)r')
+    format = _("Principal is not of the form user@REALM: '%(principal)s'")
 
 class AlreadyActive(ExecutionError):
     """
@@ -1357,7 +1327,7 @@ class HelpError(BuiltinError):
     """
 
     errno = 4101
-    format = _('no command nor help topic %(topic)r')
+    format = _("no command nor help topic '%(topic)s'")
 
 
 class LDAPError(ExecutionError):
@@ -1526,6 +1496,22 @@ class NotAllowedOnNonLeaf(ExecutionError):
     format = _('Not allowed on non-leaf entry')
 
 
+class DatabaseTimeout(DatabaseError):
+    """
+    **4211** Raised when an LDAP call times out
+
+    For example:
+
+    >>> raise DatabaseTimeout()
+    Traceback (most recent call last):
+      ...
+    DatabaseTimeout: LDAP timeout
+    """
+
+    errno = 4211
+    format = _('LDAP timeout')
+
+
 class CertificateError(ExecutionError):
     """
     **4300** Base class for Certificate execution errors (*4300 - 4399*).
@@ -1711,21 +1697,8 @@ class GenericError(PublicError):
 
 
 
-def __errors_iter():
-    """
-    Iterate through all the `PublicError` subclasses.
-    """
-    for (key, value) in globals().items():
-        if key.startswith('_') or not isclass(value):
-            continue
-        if issubclass(value, PublicError):
-            yield value
-
-public_errors = tuple(
-    sorted(__errors_iter(), key=lambda E: E.errno)
-)
+public_errors = tuple(sorted(
+    messages.iter_messages(globals(), PublicError), key=lambda E: E.errno))
 
 if __name__ == '__main__':
-    for klass in public_errors:
-        print '%d\t%s' % (klass.errno, klass.__name__)
-    print '(%d public errors)' % len(public_errors)
+    messages.print_report('public errors', public_errors)
