@@ -35,6 +35,7 @@ from errors import (ZeroArgumentError, MaxArgumentError, OverlapError,
     VersionError, OptionError, InvocationError,
     ValidationError, ConversionError)
 from ipalib import messages
+from textwrap import wrap
 
 
 RULE_FLAG = 'validation_rule'
@@ -868,7 +869,8 @@ class Command(HasParam):
         for arg in args():
             if optional and arg.required:
                 raise ValueError(
-                    '%s: required argument after optional' % arg.name
+                    '%s: required argument after optional in %s arguments %s' % (arg.name,
+                    self.name, map(lambda x: x.param_spec, args()))
                 )
             if multivalue:
                 raise ValueError(
@@ -1445,3 +1447,52 @@ class Updater(Method):
         )
 
         return self.execute(**options)
+
+
+class _AdviceOutput(object):
+
+    def __init__(self):
+        self.content = []
+        self.prefix = '# '
+        self.options = None
+
+    def comment(self, line, wrapped=True):
+        if wrapped:
+            for wrapped_line in wrap(line, 70):
+                self.content.append(self.prefix + wrapped_line)
+        else:
+            self.content.append(self.prefix + line)
+
+    def debug(self, line):
+        if self.options.verbose:
+            self.comment('DEBUG: ' + line)
+
+    def command(self, line):
+        self.content.append(line)
+
+
+class Advice(Plugin):
+    """
+    Base class for advices, plugins for ipa-advise.
+    """
+
+    options = None
+    require_root = False
+    description = ''
+
+    def __init__(self):
+        super(Advice, self).__init__()
+        self.log = _AdviceOutput()
+
+    def set_options(self, options):
+        self.options = options
+        self.log.options = options
+
+    def get_info(self):
+        """
+        This method should be overriden by child Advices.
+
+        Returns a string with instructions.
+        """
+
+        raise NotImplementedError
