@@ -102,7 +102,6 @@ a more detailed description for clarity.
 import re
 import decimal
 import base64
-import csv
 from xmlrpclib import MAXINT, MININT
 
 from types import NoneType
@@ -355,11 +354,7 @@ class Param(ReadOnly):
         parameter is not `required`
       - sortorder: used to sort a list of parameters for Command. See
         `Command.finalize()` for further information
-      - csv: this multivalue attribute is given in CSV format
-      - csv_separator: character that separates values in CSV (comma by
-        default)
-      - csv_skipspace: if true, leading whitespace will be ignored in
-        individual CSV values
+      - csv: this multivalue attribute used to be given in CSV format in CLI
     """
 
     # This is a dummy type so that most of the functionality of Param can be
@@ -393,8 +388,6 @@ class Param(ReadOnly):
         ('alwaysask', bool, False),
         ('sortorder', int, 2), # see finalize()
         ('csv', bool, False),
-        ('csv_separator', str, ','),
-        ('csv_skipspace', bool, True),
         ('option_group', unicode, None),
 
         # The 'default' kwarg gets appended in Param.__init__():
@@ -681,58 +674,6 @@ class Param(ReadOnly):
         kw.update(overrides)
         return klass(name, *self.rules, **kw)
 
-    # The following 2 functions were taken from the Python
-    # documentation at http://docs.python.org/library/csv.html
-    def __utf_8_encoder(self, unicode_csv_data):
-        for line in unicode_csv_data:
-            yield line.encode('utf-8')
-
-    def __unicode_csv_reader(self, unicode_csv_data, dialect=csv.excel, **kwargs):
-        # csv.py doesn't do Unicode; encode temporarily as UTF-8:
-        csv_reader = csv.reader(self.__utf_8_encoder(unicode_csv_data),
-                                dialect=dialect,
-                                delimiter=self.csv_separator, quotechar='"',
-                                skipinitialspace=self.csv_skipspace,
-                                **kwargs)
-        for row in csv_reader:
-            # decode UTF-8 back to Unicode, cell by cell:
-            yield [unicode(cell, 'utf-8') for cell in row]
-
-    def split_csv(self, value):
-        """Split CSV strings into individual values.
-
-        For CSV params, ``value`` is a tuple of strings. Each of these is split
-        on commas, and the results are concatenated into one tuple.
-
-        For example::
-
-            >>> param = Param('telephones', multivalue=True, csv=True)
-            >>> param.split_csv((u'1, 2', u'3', u'4, 5, 6'))
-            (u'1', u'2', u'3', u'4', u'5', u'6')
-
-        If ``value`` is not a tuple (or list), it is only split::
-
-            >>> param = Param('telephones', multivalue=True, csv=True)
-            >>> param.split_csv(u'1, 2, 3')
-            (u'1', u'2', u'3')
-
-        For non-CSV params, return the value unchanged.
-        """
-        if self.csv:
-            if type(value) not in (tuple, list):
-                value = (value,)
-            newval = []
-            for v in value:
-                if isinstance(v, basestring):
-                    lines = unicode(v).splitlines()
-                    for row in self.__unicode_csv_reader(lines):
-                        newval.extend(row)
-                else:
-                    newval.append(v)
-            return tuple(newval)
-        else:
-            return value
-
     def normalize(self, value):
         """
         Normalize ``value`` using normalizer callback.
@@ -879,10 +820,8 @@ class Param(ReadOnly):
 
     def _validate_scalar(self, value, index=None):
         if type(value) is not self.type:
-            raise ValidationError(name=self.name,
-                error='need a %r; got %r (a %r)' % (
-                    self.type, value, type(value)
-                )
+            raise TypeError(
+                TYPE_ERROR % (self.name, self.type, value, type(value))
             )
         if index is not None and type(index) is not int:
             raise TypeError(
@@ -962,8 +901,7 @@ class Param(ReadOnly):
 
     json_exclude_attrs = (
         'alwaysask', 'autofill', 'cli_name', 'cli_short_name', 'csv',
-        'csv_separator', 'csv_skipspace', 'sortorder', 'falsehoods', 'truths',
-        'version',
+        'sortorder', 'falsehoods', 'truths', 'version',
     )
 
     def __json__(self):
@@ -1103,8 +1041,7 @@ class Int(Number):
     )
 
     def __init__(self, name, *rules, **kw):
-        #pylint: disable=E1003
-        super(Number, self).__init__(name, *rules, **kw)
+        super(Int, self).__init__(name, *rules, **kw)
 
         if (self.minvalue > self.maxvalue) and (self.minvalue is not None and self.maxvalue is not None):
             raise ValueError(
@@ -1167,11 +1104,9 @@ class Int(Number):
         the exception that it allows both int and long types. The
         min/max rules handle size enforcement.
         """
-        if type(value)  not in (int, long):
-            raise ValidationError(name=self.name,
-                error='need a %r; got %r (a %r)' % (
-                    self.type, value, type(value)
-                )
+        if type(value) not in (int, long):
+            raise TypeError(
+                TYPE_ERROR % (self.name, self.type, value, type(value))
             )
         if index is not None and type(index) is not int:
             raise TypeError(
@@ -1261,7 +1196,6 @@ class Decimal(Number):
             )
 
     def _enforce_numberclass(self, value):
-        #pylint: disable=E1101
         numberclass = value.number_class()
         if numberclass not in self.numberclass:
             raise ValidationError(name=self.get_param_name(),
@@ -1285,7 +1219,7 @@ class Decimal(Number):
     def _remove_exponent(self, value):
         assert type(value) is decimal.Decimal
 
-        if not self.exponential: #pylint: disable=E1101
+        if not self.exponential:
             try:
                 # adopted from http://docs.python.org/library/decimal.html
                 value = value.quantize(decimal.Decimal(1)) \
@@ -1503,7 +1437,7 @@ class Str(Data):
         Do not allow leading/trailing spaces.
         """
         assert type(value) is unicode
-        if self.noextrawhitespace is False: #pylint: disable=E1101
+        if self.noextrawhitespace is False:
             return
         if len(value) != len(value.strip()):
             return _('Leading and trailing spaces are not allowed')
@@ -1553,7 +1487,7 @@ class IA5Str(Str):
                 if ord(value[i]) > 127:
                     raise ConversionError(name=self.get_param_name(),
                         index=index,
-                        error=_('The character \'%(char)r\' is not allowed.') %
+                        error=_('The character %(char)r is not allowed.') %
                             dict(char=value[i],)
                     )
         return super(IA5Str, self)._convert_scalar(value, index)
@@ -1866,6 +1800,23 @@ class DNParam(Param):
                                   error=ugettext(e))
         return dn
 
+
+class DeprecatedParam(Any):
+    kwargs = Param.kwargs + (
+        ('deprecate', bool, True),
+    )
+
+    def __init__(self, name, *rules, **kw):
+        if 'flags' in kw:
+            kw['flags'] = list(kw['flags']) + ['no_option']
+        else:
+            kw['flags'] = ['no_option']
+
+        super(DeprecatedParam, self).__init__(name, *rules, **kw)
+
+    def _rule_deprecate(self, _, value):
+        return _('this option is deprecated')
+
 def create_param(spec):
     """
     Create an `Str` instance from the shorthand ``spec``.
