@@ -18,13 +18,13 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import os
+
 from ipalib import api, errors
 from ipalib import Object, Command
 from ipalib import Flag, Str, IA5Str
 from ipalib.plugins.baseldap import *
 from ipalib import _, ngettext
-import ldap as _ldap
-import os
 
 __doc__ = _("""
 Automount
@@ -605,7 +605,7 @@ class automountmap_del(LDAPDelete):
         try:
             (dn_, entry_attrs) = ldap.find_entry_by_attr(
                 'automountinformation', keys[0], 'automount',
-                base_dn=self.obj.container_dn
+                base_dn=DN(self.obj.container_dn, api.env.basedn)
             )
             ldap.delete_entry(dn_)
         except errors.NotFound:
@@ -715,19 +715,19 @@ class automountkey(LDAPObject):
         # First we look with the information given, then try to search for
         # the right entry.
         try:
-            (dn, entry_attrs) = ldap.get_entry(
-                dn, ['*'], normalize=self.normalize_dn
-            )
+            (dn, entry_attrs) = ldap.get_entry(dn, ['*'])
         except errors.NotFound:
             if kwargs.get('automountinformation', False):
                 sfilter = '(&(automountkey=%s)(automountinformation=%s))' % \
                     (kwargs['automountkey'], kwargs['automountinformation'])
             else:
                 sfilter = '(automountkey=%s)' % kwargs['automountkey']
-            basedn = DN(('automountmapname', parent_keys[1]), ('cn', parent_keys[0]), self.container_dn)
+            basedn = DN(('automountmapname', parent_keys[1]),
+                        ('cn', parent_keys[0]), self.container_dn,
+                        api.env.basedn)
             attrs_list = ['*']
-            (entries, truncated) = ldap.find_entries(sfilter, attrs_list,
-                basedn, _ldap.SCOPE_ONELEVEL)
+            entries, truncated = ldap.find_entries(
+                sfilter, attrs_list, basedn, ldap.SCOPE_ONELEVEL)
             if len(entries) > 1:
                 raise errors.NotFound(reason=_('More than one entry with key %(key)s found, use --info to select specific entry.') % dict(key=pkey))
             if truncated:
@@ -889,7 +889,7 @@ class automountkey_del(LDAPDelete):
         ),
     )
     def get_options(self):
-        for option in self.takes_options:
+        for option in super(automountkey_del, self).get_options():
             if option.name == 'continue':
                 # TODO: hide for now - remove in future major release
                 yield option.clone(exclude='webui',
