@@ -400,7 +400,7 @@ class hbactest(Command):
                 ldap = self.api.Backend.ldap2
                 group_container = DN(api.env.container_group, api.env.basedn)
                 try:
-                    entries, truncated = ldap.find_entries(filter_sids, ['cn', 'memberOf'], group_container)
+                    entries, truncated = ldap.find_entries(filter_sids, ['memberof'], group_container)
                 except errors.NotFound:
                     request.user.groups = []
                 else:
@@ -409,7 +409,6 @@ class hbactest(Command):
                         memberof_dns = entry.get('memberof', [])
                         for memberof_dn in memberof_dns:
                             if memberof_dn.endswith(group_container):
-                                # this is a group object
                                 groups.append(memberof_dn[0][0].value)
                     request.user.groups = sorted(set(groups))
             else:
