@@ -66,11 +66,13 @@ There are three enrollment scenarios when enrolling a new client:
    Host Enrollment privilege.
 3. The host has been created with a one-time password.
 
-A host can only be enrolled once. If a client has enrolled and needs to
-be re-enrolled, the host entry must be removed and re-created. Note that
-re-creating the host entry will result in all services for the host being
-removed, and all SSL certificates associated with those services being
-revoked.
+
+RE-ENROLLMENT:
+
+Host that has been enrolled at some point, and lost its configuration (e.g. VM
+destroyed) can be re-enrolled.
+
+For more information, consult the manual pages for ipa-client-install.
 
 A host can optionally store information such as where it is located,
 the OS that it runs, etc.
@@ -364,22 +366,24 @@ class host(LDAPObject):
 
         return managed_hosts
 
-    def suppress_netgroup_memberof(self, entry_attrs):
+    def suppress_netgroup_memberof(self, ldap, entry_attrs):
         """
         We don't want to show managed netgroups so remove them from the
         memberofindirect list.
         """
         ng_container = DN(api.env.container_netgroup, api.env.basedn)
-        if 'memberofindirect' in entry_attrs:
-            for member in list(entry_attrs['memberofindirect']):
-                memberdn = DN(member)
-                if memberdn.endswith(ng_container):
-                    try:
-                        netgroup = api.Command['netgroup_show'](memberdn['cn'], all=True)['result']
-                        if self.has_objectclass(netgroup['objectclass'], 'mepmanagedentry'):
-                            entry_attrs['memberofindirect'].remove(member)
-                    except errors.NotFound:
-                        pass
+        for member in list(entry_attrs.get('memberofindirect', [])):
+            memberdn = DN(member)
+            if not memberdn.endswith(ng_container):
+                continue
+
+            filter = ldap.make_filter({'objectclass': 'mepmanagedentry'})
+            try:
+                ldap.get_entries(memberdn, ldap.SCOPE_BASE, filter, [''])
+            except errors.NotFound:
+                pass
+            else:
+                entry_attrs['memberofindirect'].remove(member)
 
 api.register(host)
 
@@ -421,7 +425,7 @@ class host_add(LDAPCreate):
             entry_attrs['l'] = entry_attrs['locality']
         entry_attrs['cn'] = keys[-1]
         entry_attrs['serverhostname'] = keys[-1].split('.', 1)[0]
-        if 'userpassword' not in entry_attrs and not options.get('random', False):
+        if not entry_attrs.get('userpassword', False) and not options.get('random', False):
             entry_attrs['krbprincipalname'] = 'host/%s@%s' % (
                 keys[-1], self.api.env.realm
             )
@@ -753,7 +757,7 @@ class host_mod(LDAPUpdate):
         if options.get('all', False):
             entry_attrs['managing'] = self.obj.get_managed_hosts(dn)
 
-        self.obj.suppress_netgroup_memberof(entry_attrs)
+        self.obj.suppress_netgroup_memberof(ldap, entry_attrs)
 
         convert_sshpubkey_post(ldap, dn, entry_attrs)
 
@@ -832,7 +836,7 @@ class host_find(LDAPSearch):
             set_certificate_attrs(entry_attrs)
             set_kerberos_attrs(entry_attrs, options)
             self.obj.get_password_attributes(ldap, dn, entry_attrs)
-            self.obj.suppress_netgroup_memberof(entry_attrs)
+            self.obj.suppress_netgroup_memberof(ldap, entry_attrs)
             if entry_attrs['has_password']:
                 # If an OTP is set there is no keytab, at least not one
                 # fetched anywhere.
@@ -874,7 +878,7 @@ class host_show(LDAPRetrieve):
         if options.get('all', False):
             entry_attrs['managing'] = self.obj.get_managed_hosts(dn)
 
-        self.obj.suppress_netgroup_memberof(entry_attrs)
+        self.obj.suppress_netgroup_memberof(ldap, entry_attrs)
 
         convert_sshpubkey_post(ldap, dn, entry_attrs)
 
@@ -987,7 +991,7 @@ class host_disable(LDAPQuery):
 
     def post_callback(self, ldap, dn, entry_attrs, *keys, **options):
         assert isinstance(dn, DN)
-        self.obj.suppress_netgroup_memberof(entry_attrs)
+        self.obj.suppress_netgroup_memberof(ldap, entry_attrs)
         return dn
 
 api.register(host_disable)
@@ -1001,7 +1005,7 @@ class host_add_managedby(LDAPAddMember):
 
     def post_callback(self, ldap, completed, failed, dn, entry_attrs, *keys, **options):
         assert isinstance(dn, DN)
-        self.obj.suppress_netgroup_memberof(entry_attrs)
+        self.obj.suppress_netgroup_memberof(ldap, entry_attrs)
         return (completed, dn)
 
 api.register(host_add_managedby)
@@ -1015,7 +1019,7 @@ class host_remove_managedby(LDAPRemoveMember):
 
     def post_callback(self, ldap, completed, failed, dn, entry_attrs, *keys, **options):
         assert isinstance(dn, DN)
-        self.obj.suppress_netgroup_memberof(entry_attrs)
+        self.obj.suppress_netgroup_memberof(ldap, entry_attrs)
         return (completed, dn)
 
 api.register(host_remove_managedby)
