@@ -17,13 +17,10 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from ipalib.plugins.baseldap import *
-from ipalib import api, Str, Password, DefaultFrom, _, ngettext, Object
-from ipalib.parameters import Enum
-from ipalib import Command
+from ipalib.plugins.baseldap import (LDAPObject, LDAPCreate, LDAPDelete,
+                                     LDAPRetrieve, LDAPSearch, LDAPUpdate)
+from ipalib import api, Int, Str, DeprecatedParam, StrEnum, _, ngettext
 from ipalib import errors
-from ipapython import ipautil
-from ipalib import util
 from ipapython.dn import DN
 
 if api.env.in_server and api.env.context in ['lite', 'server']:
@@ -153,6 +150,7 @@ IDs,cn=Distributed Numeric Assignment Plugin,cn=plugins,cn=config' has to be
 modified to match the new range.
 """)
 
+
 class idrange(LDAPObject):
     """
     Range object.
@@ -171,6 +169,15 @@ class idrange(LDAPObject):
     label = _('ID Ranges')
     label_singular = _('ID Range')
 
+    range_types = {
+        u'ipa-local': unicode(_('local domain range')),
+        u'ipa-ad-winsync': unicode(_('Active Directory winsync range')),
+        u'ipa-ad-trust': unicode(_('Active Directory domain range')),
+        u'ipa-ad-trust-posix': unicode(_('Active Directory trust range with '
+                                        'POSIX attributes')),
+        u'ipa-ipa-trust': unicode(_('IPA trust range')),
+                  }
+
     takes_params = (
         Str('cn',
             cli_name='name',
@@ -195,30 +202,37 @@ class idrange(LDAPObject):
         ),
         Str('ipanttrusteddomainsid?',
             cli_name='dom_sid',
+            flags=('no_update',),
             label=_('Domain SID of the trusted domain'),
         ),
         Str('ipanttrusteddomainname?',
             cli_name='dom_name',
-            flags=('no_search', 'virtual_attribute'),
+            flags=('no_search', 'virtual_attribute', 'no_update'),
             label=_('Name of the trusted domain'),
         ),
-        Str('iparangetype?',
+        StrEnum('iparangetype?',
             label=_('Range type'),
-            flags=['no_option'],
+            cli_name='type',
+            doc=(_('ID range type, one of {vals}'
+                 .format(vals=', '.join(range_types.keys())))),
+            values=tuple(range_types.keys()),
+            flags=['no_update'],
         )
     )
 
     def handle_iparangetype(self, entry_attrs, options, keep_objectclass=False):
-        if not options.get('pkey_only', False):
-            if 'ipatrustedaddomainrange' in entry_attrs.get('objectclass', []):
-                entry_attrs['iparangetype'] = [unicode(_('Active Directory domain range'))]
-            else:
-                entry_attrs['iparangetype'] = [unicode(_(u'local domain range'))]
+        if not any((options.get('pkey_only', False),
+                    options.get('raw', False))):
+            range_type = entry_attrs['iparangetype'][0]
+            entry_attrs['iparangetype'] = [self.range_types.get(range_type, None)]
+
+        # Remove the objectclass
         if not keep_objectclass:
             if not options.get('all', False) or options.get('pkey_only', False):
                 entry_attrs.pop('objectclass', None)
 
-    def check_ids_in_modified_range(self, old_base, old_size, new_base, new_size):
+    def check_ids_in_modified_range(self, old_base, old_size, new_base,
+                                    new_size):
         if new_base is None and new_size is None:
             # nothing to check
             return
@@ -231,12 +245,12 @@ class idrange(LDAPObject):
         checked_intervals = []
         low_diff = new_interval[0] - old_interval[0]
         if low_diff > 0:
-            checked_intervals.append(
-                    (old_interval[0], min(old_interval[1], new_interval[0] - 1)))
+            checked_intervals.append((old_interval[0],
+                                    min(old_interval[1], new_interval[0] - 1)))
         high_diff = old_interval[1] - new_interval[1]
         if high_diff > 0:
-            checked_intervals.append(
-                    (max(old_interval[0], new_interval[1] + 1), old_interval[1]))
+            checked_intervals.append((max(old_interval[0], new_interval[1] + 1),
+                                     old_interval[1]))
 
         if not checked_intervals:
             # range is equal or covers the entire old range, nothing to check
@@ -342,7 +356,7 @@ class idrange_add(LDAPCreate):
 
     may be given for a new ID range for the local domain while
 
-        --rid-bas
+        --rid-base
         --dom-sid
 
     must be given to add a new range for a trusted AD domain.
@@ -367,6 +381,9 @@ class idrange_add(LDAPCreate):
 
         Also ensure that secondary-rid-base is prompted for when rid-base is
         specified and vice versa, in case that dom-sid was not specified.
+
+        Also ensure that rid-base and secondary-rid-base is prompted for
+        if ipa-adtrust-install has been run on the system.
         """
 
         # dom-sid can be specified using dom-sid or dom-name options
@@ -377,24 +394,47 @@ class idrange_add(LDAPCreate):
         dom_sid_set = any(dom_id in kw for dom_id in
                           ('ipanttrusteddomainname', 'ipanttrusteddomainsid'))
 
-        rid_base_set = 'ipabaserid' in kw
-        secondary_rid_base_set = 'ipasecondarybaserid' in kw
+        rid_base = kw.get('ipabaserid', None)
+        secondary_rid_base = kw.get('ipasecondarybaserid', None)
 
-        # Prompt for RID base if domain SID / name was given
-        if dom_sid_set and not rid_base_set:
-            value = self.prompt_param(self.params['ipabaserid'])
-            kw.update(dict(ipabaserid=value))
+        def set_from_prompt(param):
+            value = self.prompt_param(self.params[param])
+            update = {param: value}
+            kw.update(update)
 
-        if not dom_sid_set:
-            # Prompt for secondary RID base if RID base was given
-            if rid_base_set and not secondary_rid_base_set:
-                value = self.prompt_param(self.params['ipasecondarybaserid'])
-                kw.update(dict(ipasecondarybaserid=value))
+        if dom_sid_set:
+            # This is a trusted range
 
-            # Symetrically, prompt for RID base if secondary RID base was given
-            if not rid_base_set and secondary_rid_base_set:
-                value = self.prompt_param(self.params['ipabaserid'])
-                kw.update(dict(ipabaserid=value))
+            # Prompt for RID base if domain SID / name was given
+            if rid_base is None:
+                set_from_prompt('ipabaserid')
+
+        else:
+            # This is a local range
+            # Find out whether ipa-adtrust-install has been ran
+            adtrust_is_enabled = api.Command['adtrust_is_enabled']()['result']
+
+            if adtrust_is_enabled:
+                # If ipa-adtrust-install has been ran, all local ranges
+                # require both RID base and secondary RID base
+
+                if rid_base is None:
+                    set_from_prompt('ipabaserid')
+
+                if secondary_rid_base is None:
+                    set_from_prompt('ipasecondarybaserid')
+
+            else:
+                # This is a local range on a server with no adtrust support
+
+                # Prompt for secondary RID base only if RID base was given
+                if rid_base is not None and secondary_rid_base is None:
+                    set_from_prompt('ipasecondarybaserid')
+
+                # Symetrically, prompt for RID base if secondary RID base was
+                # given
+                if rid_base is None and secondary_rid_base is not None:
+                    set_from_prompt('ipabaserid')
 
     def pre_callback(self, ldap, dn, entry_attrs, attrs_list, *keys, **options):
         assert isinstance(dn, DN)
@@ -420,7 +460,21 @@ class idrange_add(LDAPCreate):
                             'not be found. Please specify the SID directly '
                             'using dom-sid option.'))
 
+        # ipaNTTrustedDomainSID attribute set, this is AD Trusted domain range
         if is_set('ipanttrusteddomainsid'):
+            entry_attrs['objectclass'].append('ipatrustedaddomainrange')
+
+            # Default to ipa-ad-trust if no type set
+            if not is_set('iparangetype'):
+                entry_attrs['iparangetype'] = u'ipa-ad-trust'
+
+            if entry_attrs['iparangetype'] not in (u'ipa-ad-trust',
+                                                   u'ipa-ad-trust-posix'):
+                raise errors.ValidationError(name='ID Range setup',
+                    error=_('IPA Range type must be one of ipa-ad-trust '
+                            'or ipa-ad-trust-posix when SID of the trusted '
+                            'domain is specified.'))
+
             if is_set('ipasecondarybaserid'):
                 raise errors.ValidationError(name='ID Range setup',
                     error=_('Options dom-sid/dom-name and secondary-rid-base '
@@ -432,12 +486,26 @@ class idrange_add(LDAPCreate):
                             'be used together'))
 
             # Validate SID as the one of trusted domains
-            self.obj.validate_trusted_domain_sid(entry_attrs['ipanttrusteddomainsid'])
-            # Finally, add trusted AD domain range object class
-            entry_attrs['objectclass'].append('ipatrustedaddomainrange')
+            self.obj.validate_trusted_domain_sid(
+                                        entry_attrs['ipanttrusteddomainsid'])
 
+        # ipaNTTrustedDomainSID attribute not set, this is local domain range
         else:
-             # secondary base rid must be set if and only if base rid is set
+            entry_attrs['objectclass'].append('ipadomainidrange')
+
+            # Default to ipa-local if no type set
+            if 'iparangetype' not in entry_attrs:
+                entry_attrs['iparangetype'] = 'ipa-local'
+
+            # TODO: can also be ipa-ad-winsync here?
+            if entry_attrs['iparangetype'] in (u'ipa-ad-trust',
+                                               u'ipa-ad-trust-posix'):
+                raise errors.ValidationError(name='ID Range setup',
+                    error=_('IPA Range type must not be one of ipa-ad-trust '
+                            'or ipa-ad-trust-posix when SID of the trusted '
+                            'domain is not specified.'))
+
+            # secondary base rid must be set if and only if base rid is set
             if is_set('ipasecondarybaserid') != is_set('ipabaserid'):
                 raise errors.ValidationError(name='ID Range setup',
                     error=_('Options secondary-rid-base and rid-base must '
@@ -453,15 +521,29 @@ class idrange_add(LDAPCreate):
                             error=_("Primary RID range and secondary RID range"
                                     " cannot overlap"))
 
-            entry_attrs['objectclass'].append('ipadomainidrange')
+            # rid-base and secondary-rid-base must be set if
+            # ipa-adtrust-install has been run on the system
+            adtrust_is_enabled = api.Command['adtrust_is_enabled']()['result']
 
+            if adtrust_is_enabled and not (
+                    is_set('ipabaserid') and is_set('ipasecondarybaserid')):
+                raise errors.ValidationError(
+                    name='ID Range setup',
+                    error=_(
+                        'You must specify both rid-base and '
+                        'secondary-rid-base options, because '
+                        'ipa-adtrust-install has already been run.'
+                    )
+                )
         return dn
 
     def post_callback(self, ldap, dn, entry_attrs, *keys, **options):
         assert isinstance(dn, DN)
-        self.obj.handle_iparangetype(entry_attrs, options, keep_objectclass=True)
+        self.obj.handle_iparangetype(entry_attrs, options,
+                                     keep_objectclass=True)
         return dn
 
+
 class idrange_del(LDAPDelete):
     __doc__ = _('Delete an ID range.')
 
@@ -496,6 +578,7 @@ class idrange_del(LDAPDelete):
 
         return dn
 
+
 class idrange_find(LDAPSearch):
     __doc__ = _('Search for ranges.')
 
@@ -505,7 +588,8 @@ class idrange_find(LDAPSearch):
 
     # Since all range types are stored within separate containers under
     # 'cn=ranges,cn=etc' search can be done on a one-level scope
-    def pre_callback(self, ldap, filters, attrs_list, base_dn, scope, *args, **options):
+    def pre_callback(self, ldap, filters, attrs_list, base_dn, scope, *args,
+                     **options):
         assert isinstance(base_dn, DN)
         attrs_list.append('objectclass')
         return (filters, base_dn, ldap.SCOPE_ONELEVEL)
@@ -515,6 +599,7 @@ class idrange_find(LDAPSearch):
             self.obj.handle_iparangetype(entry, options)
         return truncated
 
+
 class idrange_show(LDAPRetrieve):
     __doc__ = _('Display information about a range.')
 
@@ -528,11 +613,17 @@ class idrange_show(LDAPRetrieve):
         self.obj.handle_iparangetype(entry_attrs, options)
         return dn
 
+
 class idrange_mod(LDAPUpdate):
     __doc__ = _('Modify ID range.')
 
     msg_summary = _('Modified ID range "%(value)s"')
 
+    takes_options = LDAPUpdate.takes_options + (
+        DeprecatedParam('ipanttrusteddomainsid?'),
+        DeprecatedParam('ipanttrusteddomainname?'),
+    )
+
     def pre_callback(self, ldap, dn, entry_attrs, attrs_list, *keys, **options):
         assert isinstance(dn, DN)
         attrs_list.append('objectclass')
@@ -586,7 +677,7 @@ class idrange_mod(LDAPUpdate):
                 self.obj.validate_trusted_domain_sid(
                     entry_attrs['ipanttrusteddomainsid'])
 
-           # Add trusted AD domain range object class, if it wasn't there
+            # Add trusted AD domain range object class, if it wasn't there
             if not 'ipatrustedaddomainrange' in old_attrs['objectclass']:
                 entry_attrs['objectclass'].append('ipatrustedaddomainrange')
 
@@ -644,6 +735,7 @@ class idrange_mod(LDAPUpdate):
         self.obj.handle_iparangetype(entry_attrs, options)
         return dn
 
+
 api.register(idrange)
 api.register(idrange_add)
 api.register(idrange_mod)
