@@ -410,27 +410,6 @@ class i18n_messages(Command):
                 "add_permission":_("Add Permission"),
                 "remove_permission": _("Remove Permission"),
             },
-            "entitle": {
-                "account": _("Account"),
-                "certificate": _("Certificate"),
-                "certificates": _("Certificates"),
-                "consume": _("Consume"),
-                "consume_entitlement": _("Consume Entitlement"),
-                "consumed": _("Consumed"),
-                "download": _("Download"),
-                "download_certificate": _("Download Certificate"),
-                "end": _("End"),
-                "import_button": _("Import"),
-                "import_certificate": _("Import Certificate"),
-                "import_message": _("Enter the Base64-encoded entitlement certificate below:"),
-                "loading": _("Loading..."),
-                "no_certificate": _("No Certificate."),
-                "product": _("Product"),
-                "register": _("Register"),
-                "registration": _("Registration"),
-                "start": _("Start"),
-                "status": _("Status"),
-            },
             "group": {
                 "details": _("Group Settings"),
                 "external": _("External"),
@@ -537,7 +516,11 @@ class i18n_messages(Command):
                 "ipasecondarybaserid": _("Secondary RID base"),
                 "type": _("Range type"),
                 "type_ad": _("Active Directory domain"),
+                "type_ad_posix": _("Active Directory domain with POSIX attributes"),
+                "type_detect": _("Detect"),
                 "type_local": _("Local domain"),
+                "type_ipa": _("IPA trust"),
+                "type_winsync": _("Active Directory winsync"),
             },
             "realmdomains": {
                 "identity": _("Realm Domains"),
@@ -617,6 +600,7 @@ class i18n_messages(Command):
                 "details": _("Trust Settings"),
                 "domain": _("Domain"),
                 "establish_using": _("Establish using"),
+                "fetch_domains": _("Fetch domains"),
                 "ipantflatname": _("Domain NetBIOS name"),
                 "ipanttrusteddomainsid": _("Domain Security Identifier"),
                 "preshared_password": _("Pre-shared password"),
