@@ -103,7 +103,7 @@ class krbtpolicy(LDAPObject):
     def get_dn(self, *keys, **kwargs):
         if keys[-1] is not None:
             return self.api.Object.user.get_dn(*keys, **kwargs)
-        return self.container_dn
+        return DN(self.container_dn, api.env.basedn)
 
 api.register(krbtpolicy)
 
@@ -176,6 +176,8 @@ class krbtpolicy_reset(LDAPQuery):
             dn = self.obj.get_dn(None)
         (dn, entry_attrs) = ldap.get_entry(dn, self.obj.default_attributes)
 
+        entry_attrs = entry_to_dict(entry_attrs, **options)
+
         if keys[-1] is not None:
             return dict(result=entry_attrs, value=keys[-1])
         return dict(result=entry_attrs, value=u'')
