@@ -88,7 +88,7 @@ class passwd(Command):
     has_output = output.standard_value
     msg_summary = _('Changed password for "%(value)s"')
 
-    def execute(self, principal, password, current_password):
+    def execute(self, principal, password, current_password, **options):
         """
         Execute the passwd operation.
 
