@@ -133,14 +133,14 @@ class permission(LDAPObject):
         Str('permissions+',
             cli_name='permissions',
             label=_('Permissions'),
-            doc=_('Comma-separated list of permissions to grant ' \
+            doc=_('Permissions to grant ' \
                 '(read, write, add, delete, all)'),
             csv=True,
         ),
         Str('attrs*',
             cli_name='attrs',
             label=_('Attributes'),
-            doc=_('Comma-separated list of attributes'),
+            doc=_('Attributes to which the permission applies'),
             csv=True,
             normalizer=lambda value: value.lower(),
             flags=('ask_create'),
@@ -330,9 +330,7 @@ class permission_mod(LDAPUpdate):
 
         # check if permission is in LDAP
         try:
-            (dn, attrs) = ldap.get_entry(
-                dn, attrs_list, normalize=self.obj.normalize_dn
-            )
+            (dn, attrs) = ldap.get_entry(dn, attrs_list)
         except errors.NotFound:
             self.obj.handle_not_found(*keys)
 
@@ -347,7 +345,7 @@ class permission_mod(LDAPUpdate):
                     except (IndexError, KeyError), e:
                         raise ValueError("expected dn starting with 'cn=' but got '%s'" % dn)
                     new_dn[0].value = options['rename']
-                    (new_dn, attrs) = ldap.get_entry(new_dn, attrs_list, normalize=self.obj.normalize_dn)
+                    (new_dn, attrs) = ldap.get_entry(new_dn, attrs_list)
                     raise errors.DuplicateEntry()
                 except errors.NotFound:
                     pass    # permission may be renamed, continue
@@ -474,10 +472,10 @@ class permission_find(LDAPSearch):
                     dn = permission['dn']
                     del permission['dn']
                     if pkey_only:
-                        new_entry = (dn, {self.obj.primary_key.name: \
-                                          permission[self.obj.primary_key.name]})
+                        pk = self.obj.primary_key.name
+                        new_entry = ldap.make_entry(dn, {pk: permission[pk]})
                     else:
-                        new_entry = (dn, permission)
+                        new_entry = ldap.make_entry(dn, permission)
 
                     if (dn, permission) not in entries:
                        if len(entries) < max_entries:
