@@ -222,7 +222,7 @@ class selinuxusermap(LDAPObject):
                     seealso,
                     self.api.Object['hbacrule'].object_class,
                     [''],
-                    self.api.Object['hbacrule'].container_dn)
+                    DN(self.api.Object['hbacrule'].container_dn, api.env.basedn))
                 seealso = dn
             except errors.NotFound:
                 raise errors.NotFound(reason=_('HBAC rule %(rule)s not found') % dict(rule=seealso))
@@ -394,7 +394,7 @@ class selinuxusermap_enable(LDAPQuery):
     msg_summary = _('Enabled SELinux User Map "%(value)s"')
     has_output = output.standard_value
 
-    def execute(self, cn):
+    def execute(self, cn, **options):
         ldap = self.obj.backend
 
         dn = self.obj.get_dn(cn)
@@ -421,7 +421,7 @@ class selinuxusermap_disable(LDAPQuery):
     msg_summary = _('Disabled SELinux User Map "%(value)s"')
     has_output = output.standard_value
 
-    def execute(self, cn):
+    def execute(self, cn, **options):
         ldap = self.obj.backend
 
         dn = self.obj.get_dn(cn)
