@@ -52,7 +52,26 @@ LDAPTLS_CACERT=/etc/ipa/ca.crt /usr/bin/ldappasswd -S -W \
 -h ipa.example.com -ZZ -D "cn=Directory Manager" \
 uid=sudo,cn=sysaccounts,cn=etc,dc=example,dc=com
 
-For more information, see the FreeIPA Documentation to Sudo.
+EXAMPLES:
+
+ Create a new rule:
+   ipa sudorule-add readfiles
+
+ Add sudo command object and add it as allowed command in the rule:
+   ipa sudocmd-add /usr/bin/less
+   ipa sudorule-add-allow-command readfiles --sudocmds /usr/bin/less
+
+ Add a host to the rule:
+   ipa sudorule-add-host readfiles --hosts server.example.com
+
+ Add a user to the rule:
+   ipa sudorule-add-user readfiles --users jsmith
+
+ Add a special Sudo rule for default Sudo server configuration:
+   ipa sudorule-add defaults
+
+ Set a default Sudo option:
+   ipa sudorule-add-option defaults --sudooption '!authenticate'
 """)
 
 topic = ('sudo', _('Commands for controlling sudo configuration'))
@@ -321,7 +340,7 @@ api.register(sudorule_show)
 class sudorule_enable(LDAPQuery):
     __doc__ = _('Enable a Sudo Rule.')
 
-    def execute(self, cn):
+    def execute(self, cn, **options):
         ldap = self.obj.backend
 
         dn = self.obj.get_dn(cn)
@@ -336,7 +355,7 @@ class sudorule_enable(LDAPQuery):
 
         return dict(result=True)
 
-    def output_for_cli(self, textui, result, cn):
+    def output_for_cli(self, textui, result, cn, **options):
         textui.print_dashed(_('Enabled Sudo Rule "%s"') % cn)
 
 api.register(sudorule_enable)
@@ -345,7 +364,7 @@ api.register(sudorule_enable)
 class sudorule_disable(LDAPQuery):
     __doc__ = _('Disable a Sudo Rule.')
 
-    def execute(self, cn):
+    def execute(self, cn, **options):
         ldap = self.obj.backend
 
         dn = self.obj.get_dn(cn)
@@ -360,7 +379,7 @@ class sudorule_disable(LDAPQuery):
 
         return dict(result=True)
 
-    def output_for_cli(self, textui, result, cn):
+    def output_for_cli(self, textui, result, cn, **options):
         textui.print_dashed(_('Disabled Sudo Rule "%s"') % cn)
 
 api.register(sudorule_disable)
@@ -605,6 +624,7 @@ api.register(sudorule_remove_runasgroup)
 class sudorule_add_option(LDAPQuery):
     __doc__ = _('Add an option to the Sudo Rule.')
 
+    has_output = output.standard_entry
     takes_options = (
         Str('ipasudoopt',
             cli_name='sudooption',
@@ -638,16 +658,16 @@ class sudorule_add_option(LDAPQuery):
             self.obj.handle_not_found(cn)
 
         attrs_list = self.obj.default_attributes
-        (dn, entry_attrs) = ldap.get_entry(
-            dn, attrs_list, normalize=self.obj.normalize_dn
-            )
+        (dn, entry_attrs) = ldap.get_entry(dn, attrs_list)
 
-        return dict(result=entry_attrs)
+        entry_attrs = entry_to_dict(entry_attrs, **options)
+
+        return dict(result=entry_attrs, value=cn)
 
     def output_for_cli(self, textui, result, cn, **options):
         textui.print_dashed(_('Added option "%(option)s" to Sudo Rule "%(rule)s"') % \
                 dict(option=options['ipasudoopt'], rule=cn))
-        super(sudorule_add_option, self).output_for_cli(textui, result, cn, options)
+        super(sudorule_add_option, self).output_for_cli(textui, result, cn, **options)
 
 
 
@@ -657,6 +677,7 @@ api.register(sudorule_add_option)
 class sudorule_remove_option(LDAPQuery):
     __doc__ = _('Remove an option from Sudo Rule.')
 
+    has_output = output.standard_entry
     takes_options = (
         Str('ipasudoopt',
             cli_name='sudooption',
@@ -693,15 +714,15 @@ class sudorule_remove_option(LDAPQuery):
             self.obj.handle_not_found(cn)
 
         attrs_list = self.obj.default_attributes
-        (dn, entry_attrs) = ldap.get_entry(
-            dn, attrs_list, normalize=self.obj.normalize_dn
-            )
+        (dn, entry_attrs) = ldap.get_entry(dn, attrs_list)
 
-        return dict(result=entry_attrs)
+        entry_attrs = entry_to_dict(entry_attrs, **options)
+
+        return dict(result=entry_attrs, value=cn)
 
     def output_for_cli(self, textui, result, cn, **options):
         textui.print_dashed(_('Removed option "%(option)s" from Sudo Rule "%(rule)s"') % \
                 dict(option=options['ipasudoopt'], rule=cn))
-        super(sudorule_remove_option, self).output_for_cli(textui, result, cn, options)
+        super(sudorule_remove_option, self).output_for_cli(textui, result, cn, **options)
 
 api.register(sudorule_remove_option)
