@@ -62,6 +62,8 @@ class Dogtag10Constants(object):
     SERVICE_PROFILE_DIR = '%s/ca/profiles/ca' % PKI_ROOT
     ALIAS_DIR = '/etc/pki/pki-tomcat/alias'
 
+    SERVICE_NAME = 'pki_tomcatd'
+
     RACERT_LINE_SEP = '\n'
 
     IPA_SERVICE_PROFILE = '%s/caIPAserviceCert.cfg' % SERVICE_PROFILE_DIR
@@ -92,6 +94,8 @@ class Dogtag9Constants(object):
     SERVICE_PROFILE_DIR = '%s/profiles/ca' % PKI_ROOT
     ALIAS_DIR = '%s/alias' % PKI_ROOT
 
+    SERVICE_NAME = 'pki-cad'
+
     RACERT_LINE_SEP = '\r\n'
 
     ADMIN_SECURE_PORT = 9445
@@ -184,7 +188,7 @@ def get_ca_certchain(ca_host=None, dogtag_constants=None):
     return chain
 
 
-def ca_status(ca_host=None):
+def ca_status(ca_host=None, use_proxy=True):
     """Return the status of the CA, and the httpd proxy in front of it
 
     The returned status can be:
@@ -194,9 +198,13 @@ def ca_status(ca_host=None):
     """
     if ca_host is None:
         ca_host = api.env.ca_host
-    # Use port 443 to test the proxy as well
+    if use_proxy:
+        # Use port 443 to test the proxy as well
+        ca_port = 443
+    else:
+        ca_port = 8443
     status, reason, headers, body = unauthenticated_https_request(
-        ca_host, 443, '/ca/admin/ca/getStatus')
+        ca_host, ca_port, '/ca/admin/ca/getStatus')
     if status == 503:
         # Service temporarily unavailable
         return reason
