@@ -425,6 +425,15 @@ class IPASimpleLDAPObject(object):
             original_dn = dn_tuple[0]
             original_attrs = dn_tuple[1]
 
+            # original_dn is None if referral instead of an entry was
+            # returned from the LDAP server, we need to skip this item
+            if original_dn is None:
+                log_msg = 'Referral entry ignored: {ref}'\
+                          .format(ref=str(original_attrs))
+                self.log.debug(log_msg)
+
+                continue
+
             ipa_entry = LDAPEntry(self, DN(original_dn))
 
             for attr, original_values in original_attrs.items():
@@ -1164,7 +1173,8 @@ class LDAPClient(object):
                     )
         return self.combine_filters(flts, rules)
 
-    def get_entries(self, base_dn, scope=None, filter=None, attrs_list=None):
+    def get_entries(self, base_dn, scope=ldap.SCOPE_SUBTREE, filter=None,
+                    attrs_list=None):
         """Return a list of matching entries.
 
         Raises an error if the list is truncated by the server
