@@ -53,6 +53,7 @@ from ipaserver.install import service
 from ipaserver.install import installutils
 from ipaserver.install import dsinstance
 from ipaserver.install import certs
+from ipaserver.install.installutils import stopped_service
 from ipaserver.plugins import ldap2
 from ipapython.ipa_log_manager import *
 
@@ -247,6 +248,11 @@ def is_step_one_done():
 
 
 class CADSInstance(service.Service):
+    """Certificate Authority DS instance
+
+    The CA DS was used with Dogtag 9. Only upgraded installations still use it.
+    Thus this class only does uninstallation.
+    """
     def __init__(self, host_name=None, realm_name=None, domain_name=None, dm_password=None, dogtag_constants=None):
         service.Service.__init__(self, "pkids",
             service_desc="directory server for the CA",
@@ -255,7 +261,7 @@ class CADSInstance(service.Service):
             autobind=service.DISABLED)
 
         self.serverid = "PKI-IPA"
-        self.realm_name = realm_name
+        self.realm = realm_name
         self.sub_dict = None
         self.domain = domain_name
         self.fqdn = host_name
@@ -265,153 +271,6 @@ class CADSInstance(service.Service):
         self.master_host = None
         self.nickname = 'Server-Cert'
         self.subject_base = None
-        if host_name and realm_name:
-            self.principal = "dogtagldap/%s@%s" % (self.fqdn, self.realm_name)
-        if realm_name:
-            self.suffix = ipautil.realm_to_suffix(self.realm_name)
-            self.__setup_sub_dict()
-        else:
-            self.suffix = DN()
-
-        if dogtag_constants is None:
-            dogtag_constants = dogtag.configured_constants()
-        self.dogtag_constants = dogtag_constants
-
-    subject_base = ipautil.dn_attribute_property('_subject_base')
-
-    def create_instance(self, realm_name, host_name, domain_name,
-                        dm_password, pkcs12_info=None, ds_port=DEFAULT_DSPORT,
-                        subject_base=None):
-        self.ds_port = ds_port
-        self.realm_name = realm_name.upper()
-        self.suffix = ipautil.realm_to_suffix(self.realm_name)
-        self.fqdn = host_name
-        self.dm_password = dm_password
-        self.domain = domain_name
-        self.pkcs12_info = pkcs12_info
-        self.subject_base = subject_base
-        self.principal = "dogtagldap/%s@%s" % (self.fqdn, self.realm_name)
-        self.__setup_sub_dict()
-
-        self.step("creating directory server user", self.__create_ds_user)
-        self.step("creating directory server instance", self.__create_instance)
-        self.step("restarting directory server", self.restart_instance)
-
-        self.start_creation(runtime=30)
-
-    def __setup_sub_dict(self):
-        server_root = dsinstance.find_server_root()
-        self.sub_dict = dict(FQDN=self.fqdn, SERVERID=self.serverid,
-                             PASSWORD=self.dm_password, SUFFIX=self.suffix,
-                             REALM=self.realm_name, USER=PKI_DS_USER,
-                             SERVER_ROOT=server_root, DOMAIN=self.domain,
-                             TIME=int(time.time()), DSPORT=self.ds_port,
-                             GROUP=dsinstance.DS_GROUP)
-
-    def __create_ds_user(self):
-        try:
-            pwd.getpwnam(PKI_DS_USER)
-            root_logger.debug("ds user %s exists" % PKI_DS_USER)
-        except KeyError:
-            root_logger.debug("adding ds user %s" % PKI_DS_USER)
-            args = ["/usr/sbin/useradd", "-g", dsinstance.DS_GROUP,
-                                         "-c", "PKI DS System User",
-                                         "-d", "/var/lib/dirsrv",
-                                         "-s", "/sbin/nologin",
-                                         "-M", "-r", PKI_DS_USER]
-            try:
-                ipautil.run(args)
-                root_logger.debug("done adding user")
-            except ipautil.CalledProcessError, e:
-                root_logger.critical("failed to add user %s" % e)
-
-    def __create_instance(self):
-        self.backup_state("serverid", self.serverid)
-
-        inf_txt = ipautil.template_str(INF_TEMPLATE, self.sub_dict)
-        root_logger.debug("writing inf template")
-        inf_fd = ipautil.write_tmp_file(inf_txt)
-        inf_txt = re.sub(r"RootDNPwd=.*\n", "", inf_txt)
-        root_logger.debug(inf_txt)
-        if ipautil.file_exists("/usr/sbin/setup-ds.pl"):
-            args = ["/usr/sbin/setup-ds.pl", "--silent", "--logfile", "-", "-f", inf_fd.name]
-            root_logger.debug("calling setup-ds.pl")
-        else:
-            args = ["/usr/bin/ds_newinst.pl", inf_fd.name]
-            root_logger.debug("calling ds_newinst.pl")
-        try:
-            ipautil.run(args)
-            root_logger.debug("completed creating ds instance")
-        except ipautil.CalledProcessError, e:
-            root_logger.critical("failed to create ds instance %s" % e)
-        inf_fd.close()
-
-    def load_pkcs12(self):
-        dirname = dsinstance.config_dirname(self.serverid)
-        dsdb = certs.CertDB(self.realm_name, nssdir=dirname)
-        if self.pkcs12_info:
-            dsdb.create_from_pkcs12(self.pkcs12_info[0], self.pkcs12_info[1])
-            server_certs = dsdb.find_server_certs()
-            if len(server_certs) == 0:
-                raise RuntimeError("Could not find a suitable server cert in import in %s" % self.pkcs12_info[0])
-
-            # We only handle one server cert
-            self.nickname = server_certs[0][0]
-            self.dercert = dsdb.get_cert_from_db(self.nickname, pem=False)
-            dsdb.track_server_cert(self.nickname, self.principal, dsdb.passwd_fname, 'restart_dirsrv %s' % self.serverid)
-
-    def create_certdb(self):
-        """
-        Create the dogtag 389-ds instance NSS certificate database. This needs
-        to be done after dogtag is installed and configured.
-        """
-        dirname = dsinstance.config_dirname(self.serverid)
-        dsdb = certs.CertDB(self.realm_name, nssdir=dirname, subject_base=self.subject_base)
-        cadb = certs.CertDB(self.realm_name, host_name=self.fqdn, subject_base=self.subject_base)
-        cadb.export_ca_cert('ipaCert', False)
-        dsdb.create_from_cacert(cadb.cacert_fname, passwd=None)
-        self.dercert = dsdb.create_server_cert("Server-Cert", self.fqdn, cadb)
-        dsdb.track_server_cert("Server-Cert", self.principal, dsdb.passwd_fname, 'restart_dirsrv %s' % self.serverid)
-        dsdb.create_pin_file()
-
-    def enable_ssl(self):
-        conn = ipaldap.IPAdmin(self.fqdn, port=DEFAULT_DSPORT)
-        conn.do_simple_bind(DN(('cn', 'directory manager')), self.dm_password)
-
-        mod = [(ldap.MOD_REPLACE, "nsSSLClientAuth", "allowed"),
-               (ldap.MOD_REPLACE, "nsSSL3Ciphers",
-                "-rsa_null_md5,+rsa_rc4_128_md5,+rsa_rc4_40_md5,+rsa_rc2_40_md5,\
-+rsa_des_sha,+rsa_fips_des_sha,+rsa_3des_sha,+rsa_fips_3des_sha,+fortezza,\
-+fortezza_rc4_128_sha,+fortezza_null,+tls_rsa_export1024_with_rc4_56_sha,\
-+tls_rsa_export1024_with_des_cbc_sha")]
-        conn.modify_s(DN(('cn', 'encryption'), ('cn', 'config')), mod)
-
-        mod = [(ldap.MOD_ADD, "nsslapd-security", "on"),
-               (ldap.MOD_ADD, "nsslapd-secureport", str(DEFAULT_DSPORT+1))]
-        conn.modify_s(DN(('cn', 'config')), mod)
-
-        entry = conn.make_entry(
-            DN(('cn', 'RSA'), ('cn', 'encryption'), ('cn', 'config')),
-            objectclass=["top", "nsEncryptionModule"],
-            cn=["RSA"],
-            nsSSLPersonalitySSL=[self.nickname],
-            nsSSLToken=["internal (software)"],
-            nsSSLActivation=["on"],
-        )
-        conn.add_entry(entry)
-
-        conn.unbind()
-
-    def restart_instance(self):
-        try:
-            ipaservices.knownservices.dirsrv.restart(self.serverid)
-            if not dsinstance.is_ds_running(self.serverid):
-                root_logger.critical("Failed to restart the directory server. See the installation log for details.")
-                sys.exit(1)
-        except Exception:
-            # TODO: roll back here?
-            root_logger.debug(traceback.format_exc())
-            root_logger.critical("Failed to restart the directory server. See the installation log for details.")
 
     def uninstall(self):
         if self.is_configured():
@@ -430,7 +289,7 @@ class CADSInstance(service.Service):
             # drop the trailing / off the config_dirname so the directory
             # will match what is in certmonger
             dirname = dsinstance.config_dirname(serverid)[:-1]
-            dsdb = certs.CertDB(self.realm_name, nssdir=dirname)
+            dsdb = certs.CertDB(self.realm, nssdir=dirname)
             dsdb.untrack_server_cert("Server-Cert")
             dsinstance.erase_ds_instance_data(serverid)
 
@@ -585,7 +444,10 @@ class CAInstance(service.Service):
             if not ipautil.dir_exists("/var/lib/pki-ca"):
                 self.step("creating pki-ca instance", self.create_instance)
             self.step("configuring certificate server instance", self.__configure_instance)
+        self.step("stopping certificate server instance to update CS.cfg", self.__stop)
         self.step("disabling nonces", self.__disable_nonce)
+        self.step("set up CRL publishing", self.__enable_crl_publish)
+        self.step("starting certificate server instance", self.__start)
         # Step 1 of external is getting a CSR so we don't need to do these
         # steps until we get a cert back from the external CA.
         if self.external != 1:
@@ -596,7 +458,6 @@ class CAInstance(service.Service):
             self.step("importing CA chain to RA certificate database", self.__import_ca_chain)
             self.step("fixing RA database permissions", self.fix_ra_perms)
             self.step("setting up signing cert profile", self.__setup_sign_profile)
-            self.step("set up CRL publishing", self.__enable_crl_publish)
             self.step("set certificate subject base", self.__set_subject_in_config)
             self.step("enabling Subject Key Identifier", self.enable_subject_key_identifier)
             self.step("enabling CRL and OCSP extensions for certificates", self.__set_crl_ocsp_extensions)
@@ -616,6 +477,13 @@ class CAInstance(service.Service):
 
         self.start_creation(runtime=210)
 
+    def __stop(self):
+        self.stop()
+
+    def __start(self):
+        self.start()
+
+
     def __spawn_instance(self):
         """
         Create and configure a new CA instance using pkispawn.
@@ -923,7 +791,8 @@ class CAInstance(service.Service):
         if update_result != 0:
             raise RuntimeError("Disabling nonces failed")
         pent = pwd.getpwnam(PKI_USER)
-        os.chown(self.dogtag_constants.CS_CFG_PATH, pent.pw_uid, pent.pw_gid)
+        os.chown(self.dogtag_constants.CS_CFG_PATH,
+                 pent.pw_uid, pent.pw_gid)
 
     def __issue_ra_cert(self):
         # The CA certificate is in the agent DB but isn't trusted
@@ -1249,6 +1118,10 @@ class CAInstance(service.Service):
         Returns a path to the CRL publishing directory
         """
         publishdir = self.dogtag_constants.CRL_PUBLISH_PATH
+
+        if not os.path.exists(publishdir):
+            os.mkdir(publishdir)
+
         os.chmod(publishdir, 0775)
         pent = pwd.getpwnam(PKI_USER)
         os.chown(publishdir, 0, pent.pw_gid)
@@ -1410,36 +1283,40 @@ class CAInstance(service.Service):
         """
         caconfig = dogtag.install_constants.CS_CFG_PATH
 
-        # Enable file publishing, disable LDAP
-        installutils.set_directive(caconfig,
-            'authz.instance.DirAclAuthz.ldap.ldapauth.authtype',
-            'SslClientAuth', quotes=False, separator='=')
-        installutils.set_directive(caconfig,
-            'authz.instance.DirAclAuthz.ldap.ldapauth.bindDN',
-            'uid=pkidbuser,ou=people,o=ipa-ca', quotes=False, separator='=')
-        installutils.set_directive(caconfig,
-            'authz.instance.DirAclAuthz.ldap.ldapauth.clientCertNickname',
-            'subsystemCert cert-pki-ca', quotes=False, separator='=')
-        installutils.set_directive(caconfig,
-            'authz.instance.DirAclAuthz.ldap.ldapconn.port',
-            str(dogtag.install_constants.DS_SECURE_PORT),
-            quotes=False, separator='=')
-        installutils.set_directive(caconfig,
-            'authz.instance.DirAclAuthz.ldap.ldapconn.secureConn',
-            'true', quotes=False, separator='=')
+        with stopped_service(self.dogtag_constants.SERVICE_NAME,
+                        instance_name=self.dogtag_constants.PKI_INSTANCE_NAME):
 
-        installutils.set_directive(caconfig, 'internaldb.ldapauth.authtype',
-            'SslClientAuth', quotes=False, separator='=')
-        installutils.set_directive(caconfig, 'internaldb.ldapauth.bindDN',
-            'uid=pkidbuser,ou=people,o=ipa-ca', quotes=False, separator='=')
-        installutils.set_directive(caconfig,
-            'internaldb.ldapauth.clientCertNickname',
-            'subsystemCert cert-pki-ca', quotes=False, separator='=')
-        installutils.set_directive(caconfig, 'internaldb.ldapconn.port',
-            str(dogtag.install_constants.DS_SECURE_PORT),
-            quotes=False, separator='=')
-        installutils.set_directive(caconfig, 'internaldb.ldapconn.secureConn',
-            'true', quotes=False, separator='=')
+            # Enable file publishing, disable LDAP
+            installutils.set_directive(caconfig,
+                'authz.instance.DirAclAuthz.ldap.ldapauth.authtype',
+                'SslClientAuth', quotes=False, separator='=')
+            installutils.set_directive(caconfig,
+                'authz.instance.DirAclAuthz.ldap.ldapauth.bindDN',
+                'uid=pkidbuser,ou=people,o=ipa-ca', quotes=False, separator='=')
+            installutils.set_directive(caconfig,
+                'authz.instance.DirAclAuthz.ldap.ldapauth.clientCertNickname',
+                'subsystemCert cert-pki-ca', quotes=False, separator='=')
+            installutils.set_directive(caconfig,
+                'authz.instance.DirAclAuthz.ldap.ldapconn.port',
+                str(dogtag.install_constants.DS_SECURE_PORT),
+                quotes=False, separator='=')
+            installutils.set_directive(caconfig,
+                'authz.instance.DirAclAuthz.ldap.ldapconn.secureConn',
+                'true', quotes=False, separator='=')
+
+            installutils.set_directive(caconfig, 'internaldb.ldapauth.authtype',
+                'SslClientAuth', quotes=False, separator='=')
+            installutils.set_directive(caconfig, 'internaldb.ldapauth.bindDN',
+                'uid=pkidbuser,ou=people,o=ipa-ca', quotes=False, separator='=')
+            installutils.set_directive(caconfig,
+                'internaldb.ldapauth.clientCertNickname',
+                'subsystemCert cert-pki-ca', quotes=False, separator='=')
+            installutils.set_directive(caconfig, 'internaldb.ldapconn.port',
+                str(dogtag.install_constants.DS_SECURE_PORT),
+                quotes=False, separator='=')
+            installutils.set_directive(caconfig,
+                 'internaldb.ldapconn.secureConn', 'true', quotes=False,
+                 separator='=')
 
     def uninstall(self):
         if self.is_configured():
@@ -1476,6 +1353,15 @@ class CAInstance(service.Service):
             root_logger.debug("Remove %s", f)
             installutils.remove_file(f)
 
+        # remove CRL directory
+        root_logger.info("Remove CRL directory")
+        if os.path.exists(self.dogtag_constants.CRL_PUBLISH_PATH):
+            try:
+                shutil.rmtree(self.dogtag_constants.CRL_PUBLISH_PATH)
+            except OSError, e:
+                root_logger.warning("Error while removing CRL publish "
+                                    "directory: %s" % e)
+
     def publish_ca_cert(self, location):
         args = ["-L", "-n", self.canickname, "-a"]
         (cert, err, returncode) = self.__run_certutil(args)
@@ -1757,7 +1643,7 @@ def install_replica_ca(config, master_ds_port, postinstall=False):
 
     if not ipautil.file_exists(cafile):
         # Replica of old "self-signed" master - skip installing CA
-        return (None, None)
+        return None
 
     if not config.setup_ca:
         # We aren't configuring the CA in this step but we still need
@@ -1766,7 +1652,7 @@ def install_replica_ca(config, master_ds_port, postinstall=False):
             dogtag_constants=dogtag.install_constants)
         ca.dm_password = config.dirman_password
         ca.subject_base = config.subject_base
-        return (ca, None)
+        return ca
 
     ca = CAInstance(config.realm_name, certs.NSS_DIR,
         dogtag_constants=dogtag.install_constants)
@@ -1780,17 +1666,6 @@ def install_replica_ca(config, master_ds_port, postinstall=False):
         pkcs12_info = (config.dir + "/dogtagcert.p12",
                        config.dir + "/dirsrv_pin.txt")
 
-    if not dogtag.install_constants.SHARED_DB:
-        cs = CADSInstance(dogtag_constants=dogtag.install_constants)
-        cs.create_instance(config.realm_name, config.host_name,
-                           config.domain_name, config.dirman_password,
-                           pkcs12_info)
-        cs.load_pkcs12()
-        cs.enable_ssl()
-        cs.restart_instance()
-    else:
-        cs = None
-
     ca = CAInstance(config.realm_name, certs.NSS_DIR,
             dogtag_constants=dogtag.install_constants)
     if postinstall:
@@ -1804,8 +1679,10 @@ def install_replica_ca(config, master_ds_port, postinstall=False):
                           master_replication_port=master_ds_port,
                           subject_base=config.subject_base)
 
+    # Restart httpd since we changed it's config and added ipa-pki-proxy.conf
+    # Without the restart, CA service status check would fail due to missing
+    # proxy
     if postinstall:
-        # Restart httpd since we changed its config
         ipaservices.knownservices.httpd.restart()
 
 
@@ -1821,18 +1698,13 @@ def install_replica_ca(config, master_ds_port, postinstall=False):
     service.print_msg("Restarting the directory and certificate servers")
     ca.stop(dogtag.install_constants.PKI_INSTANCE_NAME)
 
-    if not dogtag.install_constants.SHARED_DB:
-        ds_name = dogtag.install_constants.DS_NAME
-        ipaservices.knownservices.dirsrv.stop(ds_name)
-        ipaservices.knownservices.dirsrv.start(ds_name)
-    else:
-        ipaservices.knownservices.dirsrv.restart()
+    ipaservices.knownservices.dirsrv.restart()
 
     ca.start(dogtag.install_constants.PKI_INSTANCE_NAME)
 
-    return (ca, cs)
+    return ca
 
-def update_cert_config(nickname, cert):
+def update_cert_config(nickname, cert, dogtag_constants=None):
     """
     When renewing a CA subsystem certificate the configuration file
     needs to get the new certificate as well.
@@ -1840,6 +1712,10 @@ def update_cert_config(nickname, cert):
     nickname is one of the known nicknames.
     cert is a DER-encoded certificate.
     """
+
+    if dogtag_constants is None:
+        dogtag_constants = dogtag.configured_constants()
+
     # The cert directive to update per nickname
     directives = {'auditSigningCert cert-pki-ca': 'ca.audit_signing.cert',
                   'ocspSigningCert cert-pki-ca': 'ca.ocsp_signing.cert',
@@ -1847,10 +1723,13 @@ def update_cert_config(nickname, cert):
                   'subsystemCert cert-pki-ca': 'ca.subsystem.cert',
                   'Server-Cert cert-pki-ca': 'ca.sslserver.cert'}
 
-    installutils.set_directive(dogtag.configured_constants().CS_CFG_PATH,
-                                directives[nickname],
-                                base64.b64encode(cert),
-                                quotes=False, separator='=')
+    with stopped_service(dogtag_constants.SERVICE_NAME,
+                         instance_name=dogtag_constants.PKI_INSTANCE_NAME):
+
+        installutils.set_directive(dogtag.configured_constants().CS_CFG_PATH,
+                                    directives[nickname],
+                                    base64.b64encode(cert),
+                                    quotes=False, separator='=')
 
 def update_people_entry(uid, dercert):
     """
@@ -1914,12 +1793,7 @@ def update_people_entry(uid, dercert):
 
 if __name__ == "__main__":
     standard_logging_setup("install.log")
-    if not dogtag.install_constants.SHARED_DB:
-        cs = CADSInstance()
-        cs.create_instance(
-            "EXAMPLE.COM", "catest.example.com", "example.com", "password")
-    else:
-        ds = dsinstance.DsInstance()
+    ds = dsinstance.DsInstance()
 
     ca = CAInstance("EXAMPLE.COM", "/etc/httpd/alias")
     ca.configure_instance("catest.example.com", "example.com", "password", "password")
