@@ -29,6 +29,7 @@ import base64
 from hashlib import sha1
 
 from nss import nss
+from nss.error import NSPRError
 
 from ipapython.ipa_log_manager import root_logger
 from ipapython import dogtag
@@ -175,60 +176,25 @@ class NSSDatabase(object):
         return root_nicknames
 
     def import_pkcs12(self, pkcs12_filename, db_password_filename,
-                      pkcs_password_filename=None):
+                      pkcs12_passwd=None):
         args = ["/usr/bin/pk12util", "-d", self.secdir,
                 "-i", pkcs12_filename,
                 "-k", db_password_filename, '-v']
-        if pkcs_password_filename:
-            args = args + ["-w", pkcs_password_filename]
+        if pkcs12_passwd is not None:
+            pkcs12_passwd = pkcs12_passwd + '\n'
+            args = args + ["-w", "/dev/stdin"]
         try:
-            ipautil.run(args)
+            ipautil.run(args, stdin=pkcs12_passwd)
         except ipautil.CalledProcessError, e:
             if e.returncode == 17:
                 raise RuntimeError("incorrect password for pkcs#12 file %s" %
                     pkcs12_filename)
+            elif e.returncode == 10:
+                raise RuntimeError("Failed to open %s" % pkcs12_filename)
             else:
                 raise RuntimeError("unknown error import pkcs#12 file %s" %
                     pkcs12_filename)
 
-    def find_root_cert_from_pkcs12(self, pkcs12_fname, passwd_fname=None):
-        """Given a PKCS#12 file, try to find any certificates that do
-           not have a key. The assumption is that these are the root CAs.
-        """
-        args = ["/usr/bin/pk12util", "-d", self.secdir,
-                "-l", pkcs12_fname,
-                "-k", passwd_fname]
-        if passwd_fname:
-            args = args + ["-w", passwd_fname]
-        try:
-            (stdout, stderr, returncode) = ipautil.run(args)
-        except ipautil.CalledProcessError, e:
-            if e.returncode == 17:
-                raise RuntimeError("incorrect password for pkcs#12 file")
-            else:
-                raise RuntimeError("unknown error using pkcs#12 file")
-
-        lines = stdout.split('\n')
-
-        # A simple state machine.
-        # 1 = looking for a line starting with 'Certificate'
-        # 2 = looking for the Friendly name (nickname)
-        nicknames = []
-        state = 1
-        for line in lines:
-            if state == 2:
-                m = re.match("\W+Friendly Name: (.*)", line)
-                if m:
-                    nicknames.append( m.groups(0)[0])
-                    state = 1
-            if line == "Certificate:":
-                state = 2
-            elif not line.startswith(' '):
-                # Top-level item that is not a certificate
-                state = 1
-
-        return nicknames
-
     def trust_root_cert(self, root_nickname):
         if root_nickname[:7] == "Builtin":
             root_logger.debug(
@@ -254,8 +220,13 @@ class NSSDatabase(object):
 
         The file must contain exactly one certificate.
         """
-        with open(location) as fd:
-            certs = fd.read()
+        try:
+            with open(location) as fd:
+                certs = fd.read()
+        except IOError as e:
+            raise RuntimeError(
+                "Failed to open %s: %s" % (location, e.strerror)
+            )
 
         cert, st = find_cert_from_txt(certs)
         self.add_single_pem_cert(nickname, flags, cert)
@@ -275,6 +246,9 @@ class NSSDatabase(object):
                             "-a"],
                             stdin=cert)
 
+    def delete_cert(self, nick):
+        self.run_certutil(["-D", "-n", nick])
+
     def verify_server_cert_validity(self, nickname, hostname):
         """Verify a certificate is valid for a SSL server with given hostname
 
@@ -286,7 +260,12 @@ class NSSDatabase(object):
             certdb = nss.get_default_certdb()
             cert = nss.find_cert_from_nickname(nickname)
             intended_usage = nss.certificateUsageSSLServer
-            approved_usage = cert.verify_now(certdb, True, intended_usage)
+            try:
+                approved_usage = cert.verify_now(certdb, True, intended_usage)
+            except NSPRError, e:
+                if e.errno != -8102:
+                    raise ValueError(e.strerror)
+                approved_usage = 0
             if not approved_usage & intended_usage:
                 raise ValueError('invalid for a SSL server')
             if not cert.verify_hostname(hostname):
@@ -525,7 +504,7 @@ class CertDB(object):
         /usr/lib[64]/ipa/certmonger.
         """
         if command is not None and not os.path.isabs(command):
-            if sys.maxsize > 2**32:
+            if sys.maxsize > 2**32L:
                 libpath = 'lib64'
             else:
                 libpath = 'lib'
@@ -740,6 +719,9 @@ class CertDB(object):
                 "-f", self.passwd_fname]
         self.run_certutil(args)
 
+    def delete_cert(self, nickname):
+        self.nssdb.delete_cert(nickname)
+
     def create_pin_file(self):
         """
         This is the format of Directory Server pin files.
@@ -776,10 +758,6 @@ class CertDB(object):
 
         return root_nicknames
 
-    def find_root_cert_from_pkcs12(self, pkcs12_fname, passwd_fname=None):
-        return self.nssdb.find_root_cert_from_pkcs12(pkcs12_fname,
-                                                     passwd_fname=passwd_fname)
-
     def trust_root_cert(self, root_nickname):
         if root_nickname is None:
             root_logger.debug("Unable to identify root certificate to trust. Continuing but things are likely to fail.")
@@ -793,9 +771,9 @@ class CertDB(object):
     def find_server_certs(self):
         return self.nssdb.find_server_certs()
 
-    def import_pkcs12(self, pkcs12_fname, passwd_fname=None):
+    def import_pkcs12(self, pkcs12_fname, pkcs12_passwd=None):
         return self.nssdb.import_pkcs12(pkcs12_fname, self.passwd_fname,
-                                        pkcs_password_filename=passwd_fname)
+                                        pkcs12_passwd=pkcs12_passwd)
 
     def export_pkcs12(self, pkcs12_fname, pkcs12_pwd_fname, nickname=None):
         if nickname is None:
@@ -837,7 +815,7 @@ class CertDB(object):
         self.create_certdbs()
         self.load_cacert(cacert_fname)
 
-    def create_from_pkcs12(self, pkcs12_fname, pkcs12_pwd_fname, passwd=None,
+    def create_from_pkcs12(self, pkcs12_fname, pkcs12_passwd, passwd=None,
                            ca_file=None):
         """Create a new NSS database using the certificates in a PKCS#12 file.
 
@@ -854,7 +832,7 @@ class CertDB(object):
         self.create_noise_file()
         self.create_passwd_file(passwd)
         self.create_certdbs()
-        self.import_pkcs12(pkcs12_fname, pkcs12_pwd_fname)
+        self.import_pkcs12(pkcs12_fname, pkcs12_passwd)
         server_certs = self.find_server_certs()
         if len(server_certs) == 0:
             raise RuntimeError("Could not find a suitable server cert in import in %s" % pkcs12_fname)
@@ -877,10 +855,11 @@ class CertDB(object):
         self.create_pin_file()
         self.export_ca_cert(nickname, False)
 
-    def install_pem_from_p12(self, p12_fname, p12_pwd_fname, pem_fname):
+    def install_pem_from_p12(self, p12_fname, p12_passwd, pem_fname):
+        pwd = ipautil.write_tmp_file(p12_passwd)
         ipautil.run(["/usr/bin/openssl", "pkcs12", "-nodes",
                      "-in", p12_fname, "-out", pem_fname,
-                     "-passin", "file:" + p12_pwd_fname])
+                     "-passin", "file:" + pwd.name])
 
     def publish_ca_cert(self, location):
         shutil.copy(self.cacert_fname, location)
