@@ -42,13 +42,12 @@ from ipapython import config
 from ipalib import errors
 from ipapython.dn import DN
 from ipaserver.install import certs
+from ipapython import services as ipaservices
 
 # Used to determine install status
 IPA_MODULES = [
     'httpd', 'kadmin', 'dirsrv', 'pki-cad', 'pki-tomcatd', 'install',
     'krb5kdc', 'ntpd', 'named', 'ipa_memcached']
-if not dogtag.install_constants.SHARED_DB:
-    IPA_MODULES.append('pkids')
 
 
 class BadHostError(Exception):
@@ -258,7 +257,13 @@ def get_password(prompt):
     if os.isatty(sys.stdin.fileno()):
         return getpass.getpass(prompt)
     else:
-        return sys.stdin.readline().rstrip()
+        sys.stdout.write(prompt)
+        sys.stdout.flush()
+        line = sys.stdin.readline()
+        if not line:
+            raise EOFError()
+        return line.rstrip()
+
 
 def _read_password_default_validator(password):
     if len(password) < 8:
@@ -293,7 +298,8 @@ def read_password(user, confirm=True, validate=True, retry=True, validator=_read
                 correct = True
     except EOFError:
         return None
-    print ""
+    finally:
+        print ""
     return pwd
 
 def update_file(filename, orig, subst):
@@ -325,7 +331,7 @@ def set_directive(filename, directive, value, quotes=True, separator=' '):
     fd = open(filename)
     newfile = []
     for line in fd:
-        if directive in line:
+        if line.lstrip().startswith(directive):
             valueset = True
             if value is not None:
                 if quotes:
@@ -353,7 +359,7 @@ def get_directive(filename, directive, separator=' '):
     """
     fd = open(filename, "r")
     for line in fd:
-        if directive in line:
+        if line.lstrip().startswith(directive):
             line = line.strip()
             result = line.split(separator, 1)[1]
             result = result.strip('"')
@@ -620,14 +626,13 @@ def run_script(main_function, operation_name, log_file_name=None,
                 root_logger.info('The %s command was successful',
                     operation_name)
             else:
-                # Log at the INFO level, which is not output to the console
+                # Log at the DEBUG level, which is not output to the console
                 # (unless in debug/verbose mode), but is written to a logfile
                 # if one is open.
                 tb = sys.exc_info()[2]
-                root_logger.info('\n'.join(traceback.format_tb(tb)))
-                root_logger.info('The %s command failed, exception: %s: %s',
-                    operation_name, type(e).__name__, e)
-                exception = e
+                root_logger.debug('\n'.join(traceback.format_tb(tb)))
+                root_logger.debug('The %s command failed, exception: %s: %s',
+                                  operation_name, type(e).__name__, e)
                 if fail_message and not isinstance(e, SystemExit):
                     print fail_message
                 raise
@@ -713,7 +718,7 @@ def check_pkcs12(pkcs12_info, ca_file, hostname):
 
     Return a (server cert name, CA cert names) tuple
     """
-    pkcs12_filename, pin_filename = pkcs12_info
+    pkcs12_filename, pkcs12_passwd = pkcs12_info
     root_logger.debug('Checking PKCS#12 certificate %s', pkcs12_filename)
     db_pwd_file = ipautil.write_tmp_file(ipautil.ipa_generate_password())
     with certs.NSSDatabase() as nssdb:
@@ -722,10 +727,13 @@ def check_pkcs12(pkcs12_info, ca_file, hostname):
         # Import the CA cert first so it has a known nickname
         # (if it's present in the PKCS#12 it won't be overwritten)
         ca_cert_name = 'The Root CA'
-        nssdb.import_pem_cert(ca_cert_name, "CT,C,C", ca_file)
+        try:
+            nssdb.import_pem_cert(ca_cert_name, "CT,C,C", ca_file)
+        except (ValueError, RuntimeError) as e:
+            raise ScriptError(str(e))
 
         # Import everything in the PKCS#12
-        nssdb.import_pkcs12(pkcs12_filename, db_pwd_file.name, pin_filename)
+        nssdb.import_pkcs12(pkcs12_filename, db_pwd_file.name, pkcs12_passwd)
 
         # Check we have exactly one server cert (one with a private key)
         server_certs = nssdb.find_server_certs()
@@ -739,13 +747,20 @@ def check_pkcs12(pkcs12_info, ca_file, hostname):
         [(server_cert_name, server_cert_trust)] = server_certs
 
         # Check we have the whole cert chain & the CA is in it
-        for cert_name in nssdb.get_trust_chain(server_cert_name):
-            if cert_name == ca_cert_name:
+        trust_chain = nssdb.get_trust_chain(server_cert_name)
+        while trust_chain:
+            if trust_chain[0] == ca_cert_name:
                 break
+            trust_chain = trust_chain[1:]
         else:
             raise ScriptError(
                 '%s is not signed by %s, or the full certificate chain is not '
                 'present in the PKCS#12 file' % (pkcs12_filename, ca_file))
+        if len(trust_chain) != 2:
+            raise ScriptError(
+                'trust chain of the server certificate in %s contains %s '
+                'certificates, expected 2' %
+                (pkcs12_filename, len(trust_chain)))
 
         # Check server validity
         try:
@@ -759,21 +774,57 @@ def check_pkcs12(pkcs12_info, ca_file, hostname):
 
 
 @contextmanager
-def private_ccache():
+def private_ccache(path=None):
 
-    (desc, path) = tempfile.mkstemp(prefix='krbcc')
-    os.close(desc)
+    if path is None:
+        (desc, path) = tempfile.mkstemp(prefix='krbcc')
+        os.close(desc)
 
     original_value = os.environ.get('KRB5CCNAME', None)
 
     os.environ['KRB5CCNAME'] = path
 
-    yield
+    try:
+        yield
+    finally:
+        if original_value is not None:
+            os.environ['KRB5CCNAME'] = original_value
+        else:
+            os.environ.pop('KRB5CCNAME')
 
-    if original_value is not None:
-        os.environ['KRB5CCNAME'] = original_value
+        if os.path.exists(path):
+            os.remove(path)
+
+
+@contextmanager
+def stopped_service(service, instance_name=""):
+    """
+    Ensure that the specified service is stopped while the commands within
+    this context are executed.
+
+    Service is started at the end of the execution.
+    """
+
+    if instance_name:
+        log_instance_name = "@{instance}".format(instance=instance_name)
     else:
-        os.environ.pop('KRB5CCNAME')
+        log_instance_name = ""
 
-    if os.path.exists(path):
-        os.remove(path)
+    root_logger.debug('Ensuring that service %s%s is not running while '
+                      'the next set of commands is being executed.', service,
+                      log_instance_name)
+
+    # Figure out if the service is running, if not, yield
+    if not ipaservices.knownservices[service].is_running(instance_name):
+        root_logger.debug('Service %s%s is not running, continue.', service,
+                          log_instance_name)
+        yield
+    else:
+        # Stop the service, do the required stuff and start it again
+        root_logger.debug('Stopping %s%s.', service, log_instance_name)
+        ipaservices.knownservices[service].stop(instance_name)
+        try:
+            yield
+        finally:
+            root_logger.debug('Starting %s%s.', service, log_instance_name)
+            ipaservices.knownservices[service].start(instance_name)
