@@ -1,5 +1,4 @@
-#!/usr/bin/python
-# Authors: Rob Crittenden <rcritten@redhat.com
+# Authors: Rob Crittenden <rcritten@redhat.com>
 #
 # Copyright (C) 2013  Red Hat
 # see file 'COPYING' for use and warranty information
@@ -561,7 +560,7 @@ class Backup(admintool.AdminTool):
         if instance != 'PKI-IPA':
             return os.path.join('/var/lib/dirsrv', 'scripts-%s' % instance)
         else:
-            if sys.maxsize > 2**32:
+            if sys.maxsize > 2**32L:
                 libpath = 'lib64'
             else:
                 libpath = 'lib'
